#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2024 Canonical, Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple
from common_test import AATest, setup_all_loops

from apparmor.common import AppArmorException, AppArmorBug
from apparmor.translations import init_translation

from apparmor.rule.mount import MountRule

_ = init_translation()


class MountTestParse(AATest):

    tests = (
        #                   Rule                                                     Operation   Filesystem                Options                  Source          Destination     Audit  Deny   Allow  Comment
        ('mount -> **,',                                                    MountRule('mount',   MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  '**',           False, False, False, '')),
        ('mount options=(rw, shared) -> **,',                               MountRule('mount',   MountRule.ALL,            ('=', ('rw', 'shared')), MountRule.ALL,  '**',           False, False, False, '')),
        ('mount fstype=bpf options=rw bpf -> /sys/fs/bpf/,',                MountRule('mount',   ('=', ['bpf']),           ('=', ('rw')),           'bpf',          '/sys/fs/bpf/', False, False, False, '')),
        ('mount fstype=fuse.obex* options=rw bpf -> /sys/fs/bpf/,',         MountRule('mount',   ('=', ['fuse.obex*']),    ('=', ('rw')),           'bpf',          '/sys/fs/bpf/', False, False, False, '')),
        ('mount fstype=fuse.* options=rw bpf -> /sys/fs/bpf/,',             MountRule('mount',   ('=', ['fuse.*']),        ('=', ('rw')),           'bpf',          '/sys/fs/bpf/', False, False, False, '')),
        ('mount fstype=bpf options=(rw) random_label -> /sys/fs/bpf/,',     MountRule('mount',   ('=', ['bpf']),           ('=', ('rw')),           'random_label', '/sys/fs/bpf/', False, False, False, '')),
        ('mount,',                                                          MountRule('mount',   MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('mount fstype=(ext3, ext4),',                                      MountRule('mount',   ('=', ['ext3', 'ext4']),  MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('mount bpf,',                                                      MountRule('mount',   MountRule.ALL,            MountRule.ALL,           'bpf',          MountRule.ALL,  False, False, False, '')),
        ('mount none,',                                                     MountRule('mount',   MountRule.ALL,            MountRule.ALL,           'none',         MountRule.ALL,  False, False, False, '')),
        ('mount fstype=(procfs) none -> /foo,',                             MountRule('mount',   ('=', ['procfs']),        MountRule.ALL,           'none',         '/foo',         False, False, False, '')),
        ('mount fstype=(ext3, ext4) options=(ro),',                         MountRule('mount',   ('=', ['ext3', 'ext4']),  ('=', ('ro')),           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('mount @{mntpnt},',                                                MountRule('mount',   MountRule.ALL,            MountRule.ALL,           '@{mntpnt}',    MountRule.ALL,  False, False, False, '')),
        ('mount /a,',                                                       MountRule('mount',   MountRule.ALL,            MountRule.ALL,           '/a',           MountRule.ALL,  False, False, False, '')),
        ('mount "/a space",',                                               MountRule('mount',   MountRule.ALL,            MountRule.ALL,           '/a space',     MountRule.ALL,  False, False, False, '')),
        ('mount fstype=(ext3, ext4) /a -> /b,',                             MountRule('mount',   ('=', ['ext3', 'ext4']),  MountRule.ALL,           '/a',           '/b',           False, False, False, '')),
        ('mount fstype=(ext3, ext4) /a -> "/bar space",',                   MountRule('mount',   ('=', ['ext3', 'ext4']),  MountRule.ALL,           '/a',           '/bar space',   False, False, False, '')),
        ('mount fstype=(ext3, ext4) options=(ro, sync) /a -> /b,',          MountRule('mount',   ('=', ['ext3', 'ext4']),  ('=', ('ro', 'sync')),   '/a',           '/b',           False, False, False, '')),
        ('mount fstype=(ext3, ext4) options=(ro, sync) /a -> /b, #cmt',     MountRule('mount',   ('=', ['ext3', 'ext4']),  ('=', ('ro', 'sync')),   '/a',           '/b',           False, False, False, ' #cmt')),
        ('mount fstype=({ext3,ext4}) options in (ro, sync) /a -> /b,',      MountRule('mount',   ('=', ['{ext3,ext4}']),   ('in', ('ro', 'sync')),  '/a',           '/b',           False, False, False, '')),
        ('mount fstype in (ext3, ext4) options=(ro, sync) /a -> /b, #cmt',  MountRule('mount',   ('in', ['ext3', 'ext4']), ('=', ('ro', 'sync')),   '/a',           '/b',           False, False, False, ' #cmt')),
        ('mount fstype in (ext3, ext4) option in (ro, sync) /a, #cmt',      MountRule('mount',   ('in', ['ext3', 'ext4']), ('in', ('ro', 'sync')),  '/a',           MountRule.ALL,  False, False, False, ' #cmt')),
        ('mount fstype=(ext3, ext4) option=(ro, sync) /a -> /b, #cmt',      MountRule('mount',   ('=', ['ext3', 'ext4']),  ('=', ('ro', 'sync')),   '/a',           '/b',           False, False, False, ' #cmt')),
        ('mount fstype=fuse.revokefs-fuse options=(nosuid,nodev,rw) revokefs-fuse -> /foo-*/**/,',
                                                                            MountRule('mount',  ('=', ['fuse.revokefs-fuse']), ('=', ('nosuid', 'nodev', 'rw')),                                                # noqa: E127
                                                                                                                                                    'revokefs-fuse', '/foo-*/**/',  False, False, False, '')),  # noqa: E127
        ('mount options=(rw, rbind) {,/usr}/lib{,32,64,x32}/modules/ -> /tmp/snap.rootfs_*{,/usr}/lib/modules/,',
                                                                            MountRule('mount',   MountRule.ALL,            ('=', ('rw', 'rbind')),  '{,/usr}/lib{,32,64,x32}/modules/',  # noqa: E127
                                                                                                                                                                   '/tmp/snap.rootfs_*{,/usr}/lib/modules/',  # noqa: E127
                                                                                                                                                                                    False, False, False, '')),  # noqa: E127
        ('mount options=(runbindable, rw) -> /,',                           MountRule('mount',   MountRule.ALL,            ('=', ['runbindable', 'rw']), MountRule.ALL, '/',        False, False, False, '')),
        ('mount "" -> /,',                                                  MountRule('mount',   MountRule.ALL,            MountRule.ALL,           '',             '/',            False, False, False, '')),
        ('umount,',                                                         MountRule('umount',  MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('umount fstype=ext3,',                                             MountRule('umount',  ('=', ['ext3']),          MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('umount /a,',                                                      MountRule('umount',  MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  '/a',           False, False, False, '')),

        ('remount,',                                                        MountRule('remount', MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('remount fstype=ext4,',                                            MountRule('remount', ('=', ['ext4']),          MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('remount /b,',                                                     MountRule('remount', MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  '/b',           False, False, False, '')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(MountRule.match(rawrule))
        obj = MountRule.create_instance(rawrule)
        expected.raw_rule = rawrule.strip()
        self.assertTrue(obj.is_equal(expected, True), f'\n  {rawrule}   expected,\n  {obj.get_clean()}   returned by obj.get_clean()\n  {expected.get_clean()}   returned by expected.get_clean()')

    def test_valid_mount_changing_propagation(self):
        # Rules changing propagation type can either specify a source or a dest (these are equivalent for apparmor_parser in this specific case) but not both.
        MountRule('mount', MountRule.ALL, ('=', ('runbindable')), '/foo', MountRule.ALL)
        MountRule('mount', MountRule.ALL, ('=', ('runbindable')), MountRule.ALL, '/foo')

    def test_valid_bind_mount(self):
        # Fstype must remain empty in bind rules
        MountRule('mount', MountRule.ALL, ('=', ('bind')), '/foo', MountRule.ALL)
        MountRule('mount', MountRule.ALL, ('=', ('bind')), MountRule.ALL, '/bar')
        MountRule('mount', MountRule.ALL, ('=', ('bind')), '/foo', '/bar')


class MountTestParseInvalid(AATest):
    tests = (
        ('mount fstype=,',           AppArmorException),
        ('mount fstype=(),',         AppArmorException),
        ('mount options=(),',        AppArmorException),
        ('mount option=(invalid),',  AppArmorException),
        ('mount option=(ext3ext4),', AppArmorException),

        # mount rules with multiple 'options' are not supported by the tools yet, and when writing them, only the last 'options' would survive. Therefore MountRule intentionally raises an exception when parsing such a rule.
        ('mount options=(ro) options=(rw) fstype=ext4 -> /destination,',        AppArmorException),
        ('mount options=(ro) fstype=ext4 options=(rw) -> /destination,',        AppArmorException),
        ('mount options in (ro) options in (rw) fstype=ext4 -> /destination,',  AppArmorException),
        ('mount options in (ro) fstype=ext4 options in (rw) -> /destination,',  AppArmorException),
        ('mount options = (ro) options in (rw) fstype=ext4 -> /destination,',   AppArmorException),
        ('mount options = (ro) fstype=ext4 options in (rw) -> /destination,',   AppArmorException),

        # mount rules with multiple 'fstype' are not supported by the tools yet, and when writing them, only the last 'fstype' would survive. Therefore MountRule intentionally raises an exception when parsing such a rule.
        ('mount options=(ro) fstype=ext3 fstype=ext4 -> /destination,',             AppArmorException),
        ('mount fstype=ext3 options=(ro) fstype=ext4 -> /destination,',             AppArmorException),
        ('mount options=(ro) fstype in (ext3) fstype in (ext4) -> /destination,',   AppArmorException),
        ('mount fstype in (ext3) options=(ro) fstype in (ext4) -> /destination,',   AppArmorException),
        ('mount options=(ro) fstype in (ext3) fstype=(ext4) -> /destination,',      AppArmorException),
        ('mount fstype in (ext3) options=(ro) fstype=ext4 -> /destination,',        AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(MountRule.match(rawrule))  # the above invalid rules still match the main regex!
        with self.assertRaises(expected):
            MountRule.create_instance(rawrule)

    def test_parse_fail(self):
        with self.assertRaises(AppArmorException):
            MountRule.create_instance('foo,')

    def test_diff_non_mountrule(self):
        exp = namedtuple('exp', ('audit', 'deny', 'priority'))
        obj = MountRule('mount', ('=', ['ext4']), MountRule.ALL, MountRule.ALL, MountRule.ALL)
        with self.assertRaises(AppArmorBug):
            obj.is_equal(exp(False, False, None), False)

    def test_diff_invalid_fstype_equals_or_in(self):
        with self.assertRaises(AppArmorBug):
            MountRule('mount', ('ext3', 'ext4'), MountRule.ALL, MountRule.ALL, MountRule.ALL)  # fstype[0] should be '=' or 'in'

    def test_diff_invalid_fstype_aare(self):
        tests = [
            'mount fstype=({unclosed_regex),',
            'mount fstype=({closed}twice}),',
        ]

        for t in tests:
            with self.assertRaises(AppArmorException):
                MountRule.create_instance(t)

    def test_diff_invalid_fstype_aare_2(self):
        fslists = [
            ['invalid_{_regex'],
            ['ext4', 'invalid_}_regex'],
            ['ext4', '{invalid} {regex}']
        ]
        for fslist in fslists:
            with self.assertRaises(AppArmorException):
                MountRule('mount', ('=', fslist), MountRule.ALL, MountRule.ALL, MountRule.ALL)

    def test_diff_invalid_options_equals_or_in(self):
        with self.assertRaises(AppArmorBug):
            MountRule('mount', MountRule.ALL, ('rbind', 'rw'), MountRule.ALL, MountRule.ALL)  # fstype[0] should be '=' or 'in'

    def test_diff_invalid_options_keyword(self):
        with self.assertRaises(AppArmorException):
            MountRule('mount', MountRule.ALL, ('=', 'invalid'), MountRule.ALL, MountRule.ALL)  # fstype[0] should be '=' or 'in'

    def test_diff_fstype(self):
        obj1 = MountRule('mount', ('=', ['ext4']), MountRule.ALL, MountRule.ALL, MountRule.ALL)
        obj2 = MountRule('mount', MountRule.ALL, MountRule.ALL, MountRule.ALL, MountRule.ALL)
        self.assertFalse(obj1.is_equal(obj2, False))

    def test_diff_source(self):
        obj1 = MountRule('mount', MountRule.ALL, MountRule.ALL, '/foo', MountRule.ALL)
        obj2 = MountRule('mount', MountRule.ALL, MountRule.ALL, '/bar', MountRule.ALL)
        self.assertFalse(obj1.is_equal(obj2, False))

    def test_invalid_umount_with_source(self):
        with self.assertRaises(AppArmorException):
            MountRule('umount', MountRule.ALL, MountRule.ALL, '/foo', MountRule.ALL)  # Umount and remount shall not have a source

    def test_invalid_remount_with_source(self):
        with self.assertRaises(AppArmorException):
            MountRule('remount', MountRule.ALL, MountRule.ALL, '/foo', MountRule.ALL)

    def test_invalid_mount_changing_propagation(self):
        # Rules changing propagation type can either specify a source or a dest (these are equivalent for apparmor_parser in this specific case) but not both.
        with self.assertRaises(AppArmorException):
            MountRule('mount', MountRule.ALL, ('=', ('runbindable')), '/foo', '/bar')

        # Rules changing propagation type cannot specify a fstype.
        with self.assertRaises(AppArmorException):
            MountRule('mount', ('=', ('ext4')), ('=', ('runbindable')), MountRule.ALL, '/foo')

    def test_invalid_bind_mount(self):
        # Bind mount rules cannot specify a fstype.
        with self.assertRaises(AppArmorException):
            MountRule('mount', ('=', ('ext4')), ('=', ('bind')), MountRule.ALL, '/foo')


class MountTestGlob(AATest):
    def test_glob(self):
        globList = [(
            'mount options=(bind, rw) /home/user/Downloads/ -> /mnt/a/,',
            'mount options=(bind, rw) /home/user/Downloads/,',
            'mount options=(bind, rw) /home/user/*/,',
            'mount options=(bind, rw) /home/**/,',
            'mount options=(bind, rw),',
            'mount,',
            'mount,',
        )]
        for globs in globList:
            for i in range(len(globs) - 1):
                rule = MountRule.create_instance(globs[i])
                rule.glob()
                self.assertEqual(rule.get_clean(), globs[i + 1])


class MountTestClean(AATest):
    tests = (
        #  raw rule                                                  clean rule
        ('     mount                                                                            ,    # foo  ', 'mount, # foo'),
        ('     mount                                fstype  =  (  sysfs  )                      ,           ', 'mount fstype=(sysfs),'),
        ('     mount                                fstype  =  (  sysfs  ,  procfs  )           ,           ', 'mount fstype=(procfs, sysfs),'),
        ('     mount  options  =  (  rw  )                                                      ,           ', 'mount options=(rw),'),
        ('     mount  options  =  (  rw , noatime  )                                            ,           ', 'mount options=(noatime, rw),'),
        ('     mount                                fstype  in (  sysfs  )                      ,           ', 'mount fstype in (sysfs),'),
        ('     mount                                fstype  in (  sysfs  ,  procfs  )           ,           ', 'mount fstype in (procfs, sysfs),'),
        ('     mount  options  in (  rw  )                                                      ,           ', 'mount options in (rw),'),
        ('     mount  options  in (  rw , noatime  )                                            ,           ', 'mount options in (noatime, rw),'),
        ('     mount     none -> /foo                                                           ,           ', 'mount none -> /foo,'),
        ('     mount     ""   -> /foo                                                           ,           ', 'mount "" -> /foo,'),
        ('     mount  "/f /b" -> "/foo bar"                                                     ,           ', 'mount "/f /b" -> "/foo bar",'),
        ('     umount                                                                           ,           ', 'umount,'),
        ('     umount                                                            /foo           ,           ', 'umount /foo,'),
        ('     remount                                                                          ,           ', 'remount,'),
        ('     remount                                                           /foo           ,           ', 'remount /foo,'),
        (' deny     mount                           fstype  =  (  sysfs  )                      ,           ', 'deny mount fstype=(sysfs),'),
        ('  deny   umount                                                        /foo           ,           ', 'deny umount /foo,'),
        ('   deny  remount                                                                      ,           ', 'deny remount,'),
        ('priority =1 mount     ""   -> /foo                                                    ,           ', 'priority=1 mount "" -> /foo,'),
        ('priority=0 audit  mount  "/f /b" -> "/foo bar"                                        ,           ', 'priority=0 audit mount "/f /b" -> "/foo bar",'),
        (' priority  =  +10   umount                                                            ,           ', 'priority=10 umount,'),
        (' priority=-2    deny    umount                                         /foo           ,           ', 'priority=-2 deny umount /foo,'),
        ('priority= 32    audit  deny     remount                                               ,           ', 'priority=32 audit deny remount,'),
        (' priority = -32    remount                                             /foo           ,           ', 'priority=-32 remount /foo,'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(MountRule.match(rawrule))
        obj = MountRule.create_instance(rawrule)
        clean = obj.get_clean()
        cleanspc = obj.get_clean(1)
        raw = obj.get_raw()

        self.assertEqual(expected, clean, 'unexpected clean rule')
        self.assertEqual("  " + expected, cleanspc, 'unexpected clean rule with leading spaces')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')


class MountLogprofHeaderTest(AATest):
    tests = (
        ('mount,',                                                   [_('Operation'), _('mount'), _('Fstype'), _('ALL'),              _('Options'), _('ALL'),              _('Source'), _('ALL'), _('Destination'), _('ALL')]),
        ('mount options=(ro, nosuid) /a,',                           [_('Operation'), _('mount'), _('Fstype'), _('ALL'),              _('Options'), ('=', _('nosuid ro')), _('Source'), _('/a'),  _('Destination'), _('ALL')]),
        ('mount fstype=(ext3, ext4) options=(ro, nosuid) /a -> /b,', [_('Operation'), _('mount'), _('Fstype'), ('=', _('ext3 ext4')), _('Options'), ('=', _('nosuid ro')), _('Source'), _('/a'),  _('Destination'), _('/b')])
    )

    def _run_test(self, params, expected):
        obj = MountRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


class MountIsCoveredTest(AATest):
    def test_is_covered(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), ('=', ('ro')), '/foo/b*', '/b*')
        tests = [
            ('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), '/foo/b', '/bar'),
            ('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), '/foo/bar', '/b')
        ]
        for test in tests:
            self.assertTrue(obj.is_covered(MountRule(*test)))
            self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_covered_fs_source(self):
        obj = MountRule('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        self.assertTrue(obj.is_covered(MountRule('mount', ('=', ['ext3']), ('=', ('ro')), 'tmpfs', MountRule.ALL)))
        self.assertFalse(obj.is_equal(MountRule('mount', ('=', ['ext3']), ('=', ('ro')), 'tmpfs', MountRule.ALL)))

    def test_is_covered_aare_1(self):
        obj = MountRule('mount', ('=', ['sys*', 'fuse.*']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        tests = [
            ('mount', ('=', ['sysfs', 'fuse.s3fs']), ('=', ('ro')), 'tmpfs', MountRule.ALL),
            ('mount', ('=', ['sysfs', 'fuse.jmtpfs', 'fuse.s3fs', 'fuse.obexfs', 'fuse.obexautofs', 'fuse.fuseiso']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        ]
        for test in tests:
            self.assertTrue(obj.is_covered(MountRule(*test)))
            self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_covered_aare_2(self):
        obj = MountRule('mount', ('=', ['ext{3,4}', '{cgroup*,fuse.*}']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        tests = [
            ('mount', ('=', ['ext3']), ('=', ('ro')), 'tmpfs', MountRule.ALL),
            ('mount', ('=', ['ext3', 'ext4', 'cgroup', 'cgroup2', 'fuse.jmtpfs', 'fuse.s3fs', 'fuse.obexfs', 'fuse.obexautofs', 'fuse.fuseiso']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        ]
        for test in tests:
            self.assertTrue(obj.is_covered(MountRule(*test)))
            self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_notcovered(self):
        obj = MountRule('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), '/foo/b*', '/b*')
        tests = [
            ('mount',   ('in', ['ext3', 'ext4']),   ('=', ('ro')), '/foo/bar',     '/bar'),
            ('mount',   ('=', ['procfs', 'ext4']),  ('=', ('ro')), '/foo/bar',     '/bar'),
            ('mount',   ('=', ['ext3']),            ('=', ('rw')), '/foo/bar',     '/bar'),
            ('mount',   ('=', ['ext3', 'ext4']),    MountRule.ALL, '/foo/b*',      '/bar'),
            ('mount',   MountRule.ALL,              ('=', ('ro')), '/foo/b*',      '/bar'),
            ('mount',   ('=', ['ext3', 'ext4']),    ('=', ('ro')), '/invalid/bar', '/bar'),
            ('umount',  MountRule.ALL,              MountRule.ALL, MountRule.ALL,  '/bar'),
            ('remount', MountRule.ALL,              MountRule.ALL, MountRule.ALL,  '/bar'),
            ('mount',   ('=', ['ext3', 'ext4']),    ('=', ('ro')), 'tmpfs',        '/bar'),
            ('mount',   ('=', ['ext3', 'ext4']),    ('=', ('ro')), '/foo/b*',      '/invalid'),
        ]
        for test in tests:
            self.assertFalse(obj.is_covered(MountRule(*test)))
            self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_not_covered_fs_source(self):
        obj = MountRule('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        test = ('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), 'procfs', MountRule.ALL)
        self.assertFalse(obj.is_covered(MountRule(*test)))
        self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_not_covered_fs_options(self):
        obj = MountRule('mount', MountRule.ALL, ('=', ('ro')), 'tmpfs', MountRule.ALL)
        test = ('mount', MountRule.ALL, ('=', ('rw')), 'procfs', MountRule.ALL)
        self.assertFalse(obj.is_covered(MountRule(*test)))
        self.assertFalse(obj.is_equal(MountRule(*test)))


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import unittest

# import apparmor.aa as aa  # see the setup_aa() call for details
from common_test import AATest, setup_all_loops  # , setup_aa


class TestFoo(AATest):
    tests = (
        (0,  0),
        (42, 42),
    )

    def _run_test(self, params, expected):
        self.assertEqual(params, expected)


class TestBar(AATest):
    tests = (
        ('a', 'foo'),
        ('b', 'bar'),
        ('c', 'baz'),
    )

    def _run_test(self, params, expected):
        self.assertNotEqual(params, expected)

    def testAdditionalBarTest(self):
        self.assertEqual(1, 1)


class TestBaz(AATest):
    def AASetup(self):
        # called by setUp() - use AASetup() to avoid the need for using super(...)
        pass

    def AATeardown(self):
        # called by tearDown() - use AATeardown() to avoid the need for using super(...)
        pass

    def test_Baz_only_one_test(self):
        self.assertEqual("baz", "baz")


# if you import apparmor.aa and call init_aa() in your tests, uncomment this
# setup_aa(aa)
setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



# ----------------------------------------------------------------------
#    Copyright (C) 2013 Kshitij Gupta <kgupta8592@gmail.com>
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------
import unittest

from apparmor.common import AppArmorException
from apparmor.logparser import ReadLog
from common_test import AATest, setup_all_loops  # , setup_aa


class TestParseEvent(AATest):
    tests = ()

    def setUp(self):
        self.parser = ReadLog('', '', '')

    def test_parse_event_audit_1(self):
        event = 'type=AVC msg=audit(1345027352.096:499): apparmor="ALLOWED" operation="rename_dest" parent=6974 profile="/usr/sbin/httpd2-prefork//vhost_foo" name=2F686F6D652F7777772F666F6F2E6261722E696E2F68747470646F63732F61707061726D6F722F696D616765732F746573742F696D61676520312E6A7067 pid=20143 comm="httpd2-prefork" requested_mask="wc" denied_mask="wc" fsuid=30 ouid=30'
        parsed_event = self.parser.parse_event(event)
        self.assertEqual(parsed_event['name'], '/home/www/foo.bar.in/httpdocs/apparmor/images/test/image 1.jpg')
        self.assertEqual(parsed_event['profile'], '/usr/sbin/httpd2-prefork//vhost_foo')
        self.assertEqual(parsed_event['aamode'], 'PERMITTING')
        self.assertEqual(parsed_event['request_mask'], 'wc')

        self.assertIsNotNone(ReadLog.RE_LOG_ALL.search(event))

    def test_parse_event_audit_2(self):
        event = 'type=AVC msg=audit(1322614918.292:4376): apparmor="ALLOWED" operation="file_perm" parent=16001 profile=666F6F20626172 name="/home/foo/.bash_history" pid=17011 comm="bash" requested_mask="rw" denied_mask="rw" fsuid=0 ouid=1000'
        parsed_event = self.parser.parse_event(event)
        self.assertEqual(parsed_event['name'], '/home/foo/.bash_history')
        self.assertEqual(parsed_event['profile'], 'foo bar')
        self.assertEqual(parsed_event['aamode'], 'PERMITTING')
        self.assertEqual(parsed_event['request_mask'], 'rw')

        self.assertIsNotNone(ReadLog.RE_LOG_ALL.search(event))

    def test_parse_event_syslog_1(self):
        # from https://bugs.launchpad.net/apparmor/+bug/1399027
        event = '2014-06-09T20:37:28.975070+02:00 geeko kernel: [21028.143765] type=1400 audit(1402339048.973:1421): apparmor="ALLOWED" operation="open" profile="/home/cb/linuxtag/apparmor/scripts/hello" name="/dev/tty" pid=14335 comm="hello" requested_mask="rw" denied_mask="rw" fsuid=1000 ouid=0'
        parsed_event = self.parser.parse_event(event)
        self.assertEqual(parsed_event['name'], '/dev/tty')
        self.assertEqual(parsed_event['profile'], '/home/cb/linuxtag/apparmor/scripts/hello')
        self.assertEqual(parsed_event['aamode'], 'PERMITTING')
        self.assertEqual(parsed_event['request_mask'], 'rw')

        self.assertIsNotNone(ReadLog.RE_LOG_ALL.search(event))

    def test_parse_event_syslog_2(self):
        # from https://bugs.launchpad.net/apparmor/+bug/1399027
        event = 'Dec  7 13:18:59 rosa kernel: audit: type=1400 audit(1417954745.397:82): apparmor="ALLOWED" operation="open" profile="/home/simi/bin/aa-test" name="/usr/bin/" pid=3231 comm="ls" requested_mask="r" denied_mask="r" fsuid=1000 ouid=0'
        parsed_event = self.parser.parse_event(event)
        self.assertEqual(parsed_event['name'], '/usr/bin/')
        self.assertEqual(parsed_event['profile'], '/home/simi/bin/aa-test')
        self.assertEqual(parsed_event['aamode'], 'PERMITTING')
        self.assertEqual(parsed_event['request_mask'], 'r')

        self.assertIsNotNone(ReadLog.RE_LOG_ALL.search(event))

    def test_parse_disconnected_path(self):
        # from https://bugzilla.opensuse.org/show_bug.cgi?id=918787
        event = 'type=AVC msg=audit(1424425690.883:716630): apparmor="ALLOWED" operation="file_mmap" info="Failed name lookup - disconnected path" error=-13 profile="/sbin/klogd" name="var/run/nscd/passwd" pid=25333 comm="id" requested_mask="r" denied_mask="r" fsuid=1002 ouid=0'
        parsed_event = self.parser.parse_event(event)

        self.assertEqual(parsed_event, {
            'aamode': 'ERROR',   # aamode for disconnected paths overridden aamode in parse_event()
            'active_hat': None,
            'attr': None,
            'denied_mask': 'r',
            'error_code': 13,
            'fsuid': 1002,
            'info': 'Failed name lookup - disconnected path',
            'magic_token': 0,
            'name': 'var/run/nscd/passwd',
            'name2': None,
            'operation': 'file_mmap',
            'ouid': 0,
            'parent': 0,
            'pid': 25333,
            'profile': '/sbin/klogd',
            'request_mask': 'r',
            'resource': 'Failed name lookup - disconnected path',
            'task': 0,
            'time': 1424425690,
            'family': None,
            'protocol': None,
            'sock_type': None,
            'class': None,
        })

        self.assertIsNotNone(ReadLog.RE_LOG_ALL.search(event))

    def test_get_rule_type(self):
        rules = [
            ('mount fstype=bpf options=(rw) random_label -> /sys/fs/bpf/,', 'mount'),
            ('unix send addr=@foo{a,b} peer=(label=splat),',                'unix'),
            ('userns create, # cmt',                                        'userns'),
            ('allow /tmp/foo ra,',                                          'file'),
            ('file rwix /foo,',                                             'file'),
            ('signal set=quit peer=unconfined,',                            'signal')
        ]
        for r, exp in rules:
            self.assertEqual(self.parser.get_rule_type(r)[0], exp)

        self.assertEqual(self.parser.get_rule_type('invalid rule,'), None)

    def test_create_rule_from_ev(self):
        events = [
            ('Sep  9 12:51:50 ubuntu-desktop kernel: [ 1612.746129] type=1400 audit(1284061910.975:672): apparmor="DENIED" operation="capable" parent=2663 profile="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/syscall_setpriority" pid=7292 comm="syscall_setprio" capability=23  capname="sys_nice"',                                                    'capability',       'capability sys_nice,'),
            ('[ 1612.746129] audit: type=1400 audit(1284061910.975:672): apparmor="DENIED" operation="capable" parent=2663 profile="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/syscall_setpriority" pid=7292 comm="syscall_setprio" capability=23  capname="sys_nice"',                                                                                    'capability',       'capability sys_nice,'),
            ('Sep  9 12:51:36 ubuntu-desktop kernel: [   97.492562] audit: type=1400 audit(1431116353.523:77): apparmor="DENIED" operation="change_profile" profile="/tests/regression/apparmor/changeprofile" pid=3459 comm="changeprofile" target="/tests/regression/apparmor/rename"',                                                                           'change_profile',   'change_profile -> /tests/regression/apparmor/rename,'),
            ('Jul 31 17:10:35 dbusdev-saucy-amd64 dbus[1692]: apparmor="DENIED" operation="dbus_method_call"  bus="session" name="org.freedesktop.DBus" path="/org/freedesktop/DBus" interface="org.freedesktop.DBus" member="Hello" mask="send" pid=2922 profile="/tmp/apparmor-2.8.0/tests/regression/apparmor/dbus_service" peer_profile="unconfined"',          'dbus',             'dbus send bus=session path=/org/freedesktop/DBus interface=org.freedesktop.DBus member=Hello peer=(label=unconfined),'),
            ('Jul 31 17:11:16 dbusdev-saucy-amd64 dbus[1692]: apparmor="DENIED" operation="dbus_bind"  bus="session" name="com.apparmor.Test" mask="bind" pid=2940 profile="/tmp/apparmor-2.8.0/tests/regression/apparmor/dbus_service"',                                                                                                                           'dbus',             'dbus bind bus=session name=com.apparmor.Test,'),
            ('type=AVC msg=audit(1345027352.096:499): apparmor="ALLOWED" operation="rename_dest" parent=6974 profile="/usr/sbin/httpd2-prefork//vhost_foo" name=2F686F6D652F7777772F666F6F2E6261722E696E2F68747470646F63732F61707061726D6F722F696D616765732F746573742F696D61676520312E6A7067 pid=20143 comm="httpd2-prefork" requested_mask="wc" denied_mask="wc" fsuid=30 ouid=30',
                                                                                                                                                                                                                                                                                                                                                                    'file',             'owner "/home/www/foo.bar.in/httpdocs/apparmor/images/test/image 1.jpg" w,'),  # noqa: E127
            ('2014-06-09T20:37:28.975070+02:00 geeko kernel: [21028.143765] type=1400 audit(1402339048.973:1421): apparmor="ALLOWED" operation="open" profile="/home/cb/linuxtag/apparmor/scripts/hello" name="/dev/tty" pid=14335 comm="hello" requested_mask="rw" denied_mask="rw" fsuid=1000 ouid=0',                                                            'file',             '/dev/tty rw,'),
            ('[ 4584.703379] audit: type=1400 audit(1680266735.359:69): apparmor="DENIED" operation="uring_sqpoll" class="io_uring" profile="/root/apparmor/tests/regression/apparmor/io_uring" pid=1320 comm="io_uring" requested="sqpoll" denied="sqpoll"',                                                                                                       'io_uring',         'io_uring sqpoll,'),
            ('[ 4584.491076] audit: type=1400 audit(1680266735.147:63): apparmor="DENIED" operation="uring_override" class="io_uring" profile="/root/apparmor/tests/regression/apparmor/io_uring" pid=1193 comm="io_uring" requested="override_creds" denied="override_creds" tcontext="/root/apparmor/tests/regression/apparmor/io_uring"',                        'io_uring',         'io_uring override_creds label=/root/apparmor/tests/regression/apparmor/io_uring,'),
            ('type=AVC msg=audit(1409700640.016:547457): apparmor="DENIED" operation="mount" info="failed mntpnt match" error=-13 profile="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/mount" name="/tmp/sdtest.19033-29001-MPfz98/mountpoint/" pid=19085 comm="mount" fstype="ext2" srcname="/dev/loop0/" flags="rw, mand"',                               'mount',            'mount fstype=(ext2) options=(mand, rw) /dev/loop0/ -> /tmp/sdtest.19033-29001-MPfz98/mountpoint/,'),
            ('type=AVC msg=audit(1709108389.303:12383): apparmor="DENIED" operation="mount" class="mount" info="failed mntpnt match" error=-13 profile="/home/user/test/testmount" name="/tmp/foo/" pid=14155 comm="testmount" flags="ro, remount"',                                                                                                                'mount',            'mount options=(remount, ro) -> /tmp/foo/,'),
            ('type=AVC msg=audit(1709025786.045:43147): apparmor="DENIED" operation="umount" class="mount" profile="/home/user/test/testmount" name="/mnt/a/" pid=26697 comm="testmount"',                                                                                                                                                                          'mount',            'umount /mnt/a/,'),
            ('Apr 05 19:36:19 ubuntu kernel: audit: type=1400 audit(1649187379.660:255): apparmor="DENIED" operation="create" profile="/root/apparmor/tests/regression/apparmor/posix_mq_rcv" name="/queuename" pid=791 comm="posix_mq_rcv" requested="create" denied="create" class="posix_mqueue" fsuid=0 ouid=0',                                                'mqueue',           'mqueue create type=posix /queuename,'),
            ('Apr 05 19:36:29 ubuntu kernel: audit: type=1400 audit(1649187389.828:262): apparmor="DENIED" operation="open" profile="/root/apparmor/tests/regression/apparmor/posix_mq_rcv" name="/queuename" pid=848 comm="posix_mq_rcv" requested="read create" denied="read" class="posix_mqueue" fsuid=0 ouid=0',                                               'mqueue',           'mqueue read type=posix /queuename,'),
            ('Apr  5 19:30:56 precise-amd64 kernel: [153073.826757] type=1400 audit(1308766940.698:3704): apparmor="DENIED" operation="sendmsg" parent=24737 profile="/usr/bin/evince-thumbnailer" pid=24743 comm="evince-thumbnai" laddr=192.168.66.150 lport=765 faddr=192.168.66.200 fport=2049 family="inet" sock_type="stream" protocol=6',                    'network',          'network inet stream ip=192.168.66.150 port=765 peer=(ip=192.168.66.200 port=2049),'),
            ('Apr  5 19:31:04 precise-amd64 kernel: [153073.826757] type=1400 audit(1308766940.698:3704): apparmor="DENIED" operation="sendmsg" parent=24737 profile="/usr/bin/evince-thumbnailer" pid=24743 comm="evince-thumbnai" lport=765 fport=2049 family="inet" sock_type="stream" protocol=6',                                                              'network',          'network inet stream port=765 peer=(port=2049),'),
            ('type=AVC msg=audit(1409700678.384:547594): apparmor="DENIED" operation="pivotroot" profile="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/pivot_root" name="/tmp/sdtest.21082-7446-EeefO6/new_root/" pid=21162 comm="pivot_root" srcname="/tmp/sdtest.21082-7446-EeefO6/new_root/put_old/"',                                                    'pivot_root',       'pivot_root oldroot=/tmp/sdtest.21082-7446-EeefO6/new_root/put_old/ /tmp/sdtest.21082-7446-EeefO6/new_root/,'),
            ('type=AVC msg=audit(1409700683.304:547661): apparmor="DENIED" operation="ptrace" profile="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace" pid=22465 comm="ptrace" requested_mask="tracedby" denied_mask="tracedby" peer="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace"',                                                     'ptrace',           'ptrace tracedby peer=/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace,'),
            ('type=AVC msg=audit(1409438250.564:201): apparmor="DENIED" operation="signal" profile="/usr/bin/pulseaudio" pid=2531 comm="pulseaudio" requested_mask="send" denied_mask="send" signal=term peer="/usr/bin/pulseaudio///usr/lib/pulseaudio/pulse/gconf-helper"',                                                                                       'signal',           'signal send set=term peer=/usr/bin/pulseaudio///usr/lib/pulseaudio/pulse/gconf-helper,'),
            ('type=AVC msg=audit(1711454639.955:322): apparmor="DENIED" operation="connect" class="net" profile="/home/user/test/client.py" pid=80819 comm="client.py" family="unix" sock_type="stream" protocol=0 requested="send receive connect" denied="send receive connect" addr=none peer_addr="@test_abstract_socket" peer="/home/user/test/server.py"',    'unix',             'unix (connect, receive, send) type=stream peer=(addr=@test_abstract_socket),'),
            ('type=AVC msg=audit(1711214183.107:298): apparmor="DENIED" operation="connect" class="net" profile="/home/user/test/client.py" pid=65262 comm="server.py" family="unix" sock_type="stream" protocol=0 requested="send receive accept" denied="send accept" addr="@test_abstract_socket" peer_addr=none peer="unconfined"',                             'unix',             'unix (accept, send) type=stream addr=@test_abstract_socket,'),
            ('[  176.385388] audit: type=1400 audit(1666891380.570:78): apparmor="DENIED" operation="userns_create" class="namespace" profile="/usr/bin/userns_child_exec" pid=1785 comm="userns_child_ex" requested="userns_create" denied="userns_create"',                                                                                                       'userns',           'userns create,'),
            ('[  429.272003] audit: type=1400 audit(1720613712.153:168): apparmor="AUDIT" operation="userns_create" class="namespace" info="Userns create - transitioning profile" profile="unconfined" pid=5630 comm="unshare" requested="userns_create" target="unprivileged_userns" execpath="/usr/bin/unshare"',                                                'userns',           'userns create,'),
        ]
        for ev, expected_type, expected_rule in events:
            parsed_event = self.parser.parse_event(ev)
            r = self.parser.create_rule_from_ev(parsed_event)
            self.assertIsNotNone(r)
            clean_rule = r.get_clean()
            self.assertEqual(self.parser.get_rule_type(clean_rule)[0], expected_type)
            self.assertEqual(expected_rule, clean_rule)


class TestParseEventForTreeInvalid(AATest):
    tests = (
        ('type=AVC msg=audit(1556742870.707:3614): apparmor="ALLOWED" operation="open" profile="/bin/hello" name="/dev/tty" pid=12856 comm="hello" requested_mask="wr" denied_mask="foo" fsuid=1000 ouid=0',   AppArmorException),  # invalid file permissions "foo"
        ('type=AVC msg=audit(1556742870.707:3614): apparmor="ALLOWED" operation="open" profile="/bin/hello" name="/dev/tty" pid=12856 comm="hello" requested_mask="wr" denied_mask="wr::w" fsuid=1000 ouid=0', AppArmorException),  # "wr::w" mixes owner and other
    )

    def setUp(self):
        self.parser = ReadLog('', '', '')

    def _fake_profile_exists(self, program):
        return True

    def test_invalid_create_rule_from_ev(self):
        events = [
            'Jul 31 17:11:16 dbusdev-saucy-amd64 dbus[1692]: apparmor="DENIED" operation="invalid"  bus="session" name="com.apparmor.Test" mask="bind" pid=2940 profile="/tmp/apparmor-2.8.0/tests/regression/apparmor/dbus_service"',  # operation is invalid
            'type=AVC msg=audit(1709108389.303:12383): apparmor="DENIED" operation="mount" class="mount" info="failed mntpnt match" error=-13 profile="/home/user/test/testmount" name="/tmp/foo/" pid=14155 comm="testmount" flags="invalid"',  # invalid flag
            '[  176.385388] audit: type=1400 audit(1666891380.570:78): apparmor="DENIED" operation="userns_create" class="namespace" profile="/usr/bin/userns_child_exec" pid=1785 comm="userns_child_ex" requested="userns_create" denied="invalid"'  # invalid denied
        ]
        for ev in events:
            parsed_event = self.parser.parse_event(ev)
            r = self.parser.create_rule_from_ev(parsed_event)
            self.assertEqual(r, None)

    def _run_test(self, params, expected):
        self.parser.profile_exists = self._fake_profile_exists  # inject fake function that always returns True - much easier than handing over a ProfileList object to __init__
        parsed_event = self.parser.parse_event(params)
        with self.assertRaises(expected):
            self.parser.parse_event_for_tree(parsed_event)


setup_all_loops(__name__)
if __name__ == "__main__":
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2021 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import unittest

from apparmor.common import AppArmorBug
from apparmor.notify import get_last_login_timestamp, get_last_login_timestamp_wtmp, sane_timestamp
from common_test import AATest, setup_all_loops


class TestGet_last_login_timestamp(AATest):
    tests = (
        #  wtmp file         lastlog2 db         user           expected login timestamp
        (('wtmp-x86_64',    'lastlog2.db',     'root'),         1723749426),  # Thu Aug 15 19:17:06 UTC 2024
        (('wtmp-x86_64',    'lastlog2.db',     'whoever'),      0),
        (('wtmp-x86_64',    'lastlog2.db',     'cb'),           1726995194),  # Sun Sep 22 08:53:14 UTC 2024
        (('wtmp-x86_64',    'lastlog2.db',     'sddm'),         1721084423),  # Mon Jul 15 23:00:23 UTC 2024

        (('wtmp-x86_64',    'does-not-exist',   'root'),        1635070346),  # Sun Oct 24 12:12:26 CEST 2021
        (('wtmp-x86_64',    'does-not-exist',   'whoever'),     0),

        (('wtmp-s390x',     'lastlog2.db',      'root'),        1723749426),  # Thu Aug 15 19:17:06 UTC 2024
        (('wtmp-s390x',     'lastlog2.db',      'whoever'),     0),
        (('wtmp-s390x',     'does-not-exist',   'linux1'),      1626368772),  # Thu Jul 15 19:06:12 CEST 2021
        (('wtmp-s390x',     'does-not-exist',   'whoever'),     0),

        (('wtmp-aarch64',   'lastlog2.db',      'whoever'),     0),
        (('wtmp-aarch64',   'does-not-exist',   'guillaume'),   1611562789),  # Mon Jan 25 09:19:49 CET 2021
        (('wtmp-aarch64',   'does-not-exist',   'whoever'),     0),

        (('wtmp-truncated', 'does-not-exist',   'root'),        0),
        (('wtmp-truncated', 'does-not-exist',   'whoever'),     0),
    )

    def _run_test(self, params, expected):
        wtmpdb, lastlog2_db, user = params
        wtmpdb = 'wtmp-examples/' + wtmpdb
        lastlog2_db = 'wtmp-examples/' + lastlog2_db
        self.assertEqual(get_last_login_timestamp(user, wtmpdb, lastlog2_db), expected)

    def test_date_1999(self):
        with self.assertRaises(AppArmorBug):
            # wtmp-x86_64-past is hand-edited to Thu Dec 30 00:00:00 CET 1999, which is outside the expected data range
            get_last_login_timestamp('root', 'wtmp-examples/wtmp-x86_64-past', 'wtmp-examples/does-not-exist')


class TestSane_timestamp(AATest):
    tests = (
        (2524704400, False),  # Sun Jan  2 03:46:40 CET 2050
        (944780400, False),  # Fri Dec 10 00:00:00 CET 1999
        (1635026400, True),  # Sun Oct 24 00:00:00 CEST 2021
    )

    def _run_test(self, params, expected):
        self.assertEqual(sane_timestamp(params), expected)


class TestGet_last_login_timestamp_wtmp(AATest):
    tests = (
        (('wtmp-x86_64',    'root'),      1635070346),  # Sun Oct 24 12:12:26 CEST 2021
        (('wtmp-x86_64',    'whoever'),   0),
        (('wtmp-s390x',     'root'),      1626368763),  # Thu Jul 15 19:06:03 CEST 2021
        (('wtmp-s390x',     'linux1'),    1626368772),  # Thu Jul 15 19:06:12 CEST 2021
        (('wtmp-s390x',     'whoever'),   0),
        (('wtmp-aarch64',   'guillaume'), 1611562789),  # Mon Jan 25 09:19:49 CET 2021
        (('wtmp-aarch64',   'whoever'),   0),
        (('wtmp-truncated', 'root'),      0),
        (('wtmp-truncated', 'whoever'),   0),
    )

    def _run_test(self, params, expected):
        filename, user = params
        filename = 'wtmp-examples/' + filename
        self.assertEqual(get_last_login_timestamp_wtmp(user, filename), expected)

    def test_date_1999(self):
        with self.assertRaises(AppArmorBug):
            # wtmp-x86_64-past is hand-edited to Thu Dec 30 00:00:00 CET 1999, which is outside the expected data range
            get_last_login_timestamp_wtmp('root', 'wtmp-examples/wtmp-x86_64-past')


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2018 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import os
import shutil
import unittest

import apparmor.aa
from apparmor.common import AppArmorBug, AppArmorException
from apparmor.profile_list import ProfileList
from apparmor.profile_storage import ProfileStorage
from apparmor.aare import AARE
from apparmor.rule.abi import AbiRule
from apparmor.rule.alias import AliasRule
from apparmor.rule.boolean import BooleanRule
from apparmor.rule.include import IncludeRule
from apparmor.rule.variable import VariableRule
from common_test import AATest, setup_aa, setup_all_loops, write_file, skip_active_profiles


class TestAdd_profile(AATest):
    def AASetup(self):
        self.pl = ProfileList()
        self.dummy_profile = ProfileStorage('TEST DUMMY', 'AATest_no_file', 'TEST')

    def testEmpty(self):
        self.assertEqual(self.pl.profile_names, {})
        self.assertEqual(self.pl.attachments, {})
        self.assertEqual(str(self.pl), "\n".join(['', '<ProfileList>', '', '</ProfileList>', '']))

    def testAdd_profile_1(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)
        self.assertEqual(self.pl.profile_names, {'foo': '/etc/apparmor.d/bin.foo'})
        self.assertEqual(self.pl.attachments, {'/bin/foo': {'f': '/etc/apparmor.d/bin.foo', 'p': 'foo', 're': AARE('/bin/foo', True)}})
        self.assertEqual(self.pl.profiles_in_file('/etc/apparmor.d/bin.foo'), ['foo'])
        self.assertEqual(str(self.pl), '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')

    def testAdd_profile_2(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', None, '/bin/foo', self.dummy_profile)
        self.assertEqual(self.pl.profile_names, {})
        self.assertEqual(self.pl.attachments, {'/bin/foo': {'f': '/etc/apparmor.d/bin.foo', 'p': '/bin/foo', 're': AARE('/bin/foo', True)}})
        self.assertEqual(self.pl.profiles_in_file('/etc/apparmor.d/bin.foo'), ['/bin/foo'])
        self.assertEqual(str(self.pl), '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')

    def testAdd_profile_3(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', None, self.dummy_profile)
        self.assertEqual(self.pl.profile_names, {'foo': '/etc/apparmor.d/bin.foo'})
        self.assertEqual(self.pl.attachments, {})
        self.assertEqual(self.pl.profiles_in_file('/etc/apparmor.d/bin.foo'), ['foo'])
        self.assertEqual(str(self.pl), '\n<ProfileList>\n/etc/apparmor.d/bin.foo\n</ProfileList>\n')

    def testAdd_profileError_1(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_profile('', 'foo', '/bin/foo', self.dummy_profile)  # no filename

    def testAdd_profileError_2(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_profile('/etc/apparmor.d/bin.foo', None, None, self.dummy_profile)  # neither attachment nor profile name

    def testAdd_profileError_list_nonexisting_file(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', None, self.dummy_profile)
        with self.assertRaises(AppArmorBug):
            self.pl.profiles_in_file('/etc/apparmor.d/not.found')  # different filename

    def testAdd_profileError_twice_1(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)
        with self.assertRaises(AppArmorException):
            self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)

    def testAdd_profileError_twice_2(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)
        with self.assertRaises(AppArmorException):
            self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', None, self.dummy_profile)

    def testAdd_profileError_twice_3(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', None, '/bin/foo', self.dummy_profile)
        with self.assertRaises(AppArmorException):
            self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)

    def testAdd_profileError_twice_4(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', None, '/bin/foo', self.dummy_profile)
        with self.assertRaises(AppArmorException):
            self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)

    def testAdd_profileError_twice_5(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', None, self.dummy_profile)
        with self.assertRaises(AppArmorException):
            self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)

    def testAdd_profileError_wrong_prof_type(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', 'wrong_type')


class TestFilename_from_profile_name(AATest):
    tests = (
        ('foo',         '/etc/apparmor.d/bin.foo'),
        ('/bin/foo',    None),
        ('bar',         None),
        ('/usr{,{/lib,/lib32,/lib64}/wine}/bin/wine{,-preloader,server}{,-staging-*,-vanilla-*}', '/etc/apparmor.d/usr.bin.wine'),
        ('/usr/lib/wine/bin/wine-preloader-staging-foo',                                          None),  # no AARE matching for profile names
    )

    def AASetup(self):
        self.pl = ProfileList()
        self.dummy_profile = ProfileStorage('TEST DUMMY', 'AATest_no_file', 'TEST')
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)
        self.pl.add_profile(
            '/etc/apparmor.d/usr.bin.wine',
            '/usr{,{/lib,/lib32,/lib64}/wine}/bin/wine{,-preloader,server}{,-staging-*,-vanilla-*}',
            '/usr{,{/lib,/lib32,/lib64}/wine}/bin/wine{,-preloader,server}{,-staging-*,-vanilla-*}',
            self.dummy_profile)

    def _run_test(self, params, expected):
        self.assertEqual(self.pl.filename_from_profile_name(params), expected)


class TestFilename_from_attachment(AATest):
    tests = (
        ('/bin/foo',    '/etc/apparmor.d/bin.foo'),
        ('/bin/baz',    '/etc/apparmor.d/bin.baz'),
        ('/bin/foobar', '/etc/apparmor.d/bin.foobar'),
        ('@{foo}',      None),  # XXX variables not supported yet (and @{foo} isn't defined in this test)
        ('/bin/404',    None),
        ('/usr{,{/lib,/lib32,/lib64}/wine}/bin/wine{,-preloader,server}{,-staging-*,-vanilla-*}', '/etc/apparmor.d/usr.bin.wine'),  # XXX should this really match, or should attachment matching only use AARE?
        ('/usr/lib/wine/bin/wine-preloader-staging-foo',                                          '/etc/apparmor.d/usr.bin.wine'),  # AARE match
    )

    def AASetup(self):
        self.pl = ProfileList()
        self.dummy_profile = ProfileStorage('TEST DUMMY', 'AATest_no_file', 'TEST')
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo', '/bin/foo', self.dummy_profile)
        self.pl.add_profile('/etc/apparmor.d/bin.baz', 'baz', '/bin/ba*', self.dummy_profile)
        self.pl.add_profile('/etc/apparmor.d/bin.foobar', 'foobar', '/bin/foo{bar,baz}', self.dummy_profile)
        self.pl.add_profile('/etc/apparmor.d/bin.asdf', None, '/bin/asdf', self.dummy_profile)
        self.pl.add_profile(
            '/etc/apparmor.d/usr.bin.wine',
            'wine',
            '/usr{,{/lib,/lib32,/lib64}/wine}/bin/wine{,-preloader,server}{,-staging-*,-vanilla-*}',
            self.dummy_profile)

    def _run_test(self, params, expected):
        self.assertEqual(self.pl.filename_from_attachment(params), expected)

    def test_non_path_attachment(self):
        with self.assertRaises(AppArmorBug):
            self.pl.filename_from_attachment('foo')


class TestProfile_from_attachment(TestFilename_from_attachment):
    # uses AASetup from TestFilename_from_attachment
    tests = (
        ('/bin/foo',    'foo'),
        ('/bin/baz',    'baz'),
        ('/bin/foobar', 'foobar'),
        ('/bin/asdf',   '/bin/asdf'),
        ('@{foo}',      None),  # XXX variables not supported yet (and @{foo} isn't defined in this test)
        ('/bin/404',    None),
        ('/usr{,{/lib,/lib32,/lib64}/wine}/bin/wine{,-preloader,server}{,-staging-*,-vanilla-*}', 'wine'),  # XXX should this really match, or should attachment matching only use AARE?
        ('/usr/lib/wine/bin/wine-preloader-staging-foo',                                          'wine'),  # AARE match
    )

    def _run_test(self, params, expected):
        self.assertEqual(self.pl.profile_from_attachment(params), expected)

    def test_non_path_attachment(self):
        with self.assertRaises(AppArmorBug):
            self.pl.profile_from_attachment('foo')


class TestAdd_inc_ie(AATest):
    def AASetup(self):
        self.pl = ProfileList()

    def testAdd_inc_ie_1(self):
        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule('tunables/global', False, True))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['include <tunables/global>', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['include <tunables/global>', ''])

    def testAdd_inc_ie_2(self):
        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule('tunables/global', False, True))
        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule('tunables/dovecot', False, True))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['include <tunables/global>', 'include <tunables/dovecot>', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['include <tunables/global>', 'include <tunables/dovecot>', ''])

    def testAdd_inc_ie_error_1(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', 'tunables/global')  # str instead of IncludeRule
        self.assertEqual(list(self.pl.files.keys()), [])

    def test_dedup_inc_ie_1(self):
        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.create_instance('include <tunables/global>'))
        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.create_instance('#include if exists <tunables/global>  # comment'))
        self.pl.add_inc_ie('/etc/apparmor.d/bin.foo', IncludeRule.create_instance('   #include         <tunables/global>    '))
        deleted = self.pl.delete_preamble_duplicates('/etc/apparmor.d/bin.foo')
        self.assertEqual(deleted, 2)
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['include <tunables/global>', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['include <tunables/global>', ''])

    def test_dedup_error_1(self):
        with self.assertRaises(AppArmorBug):
            self.pl.delete_preamble_duplicates('/file/not/found')
        self.assertEqual(list(self.pl.files.keys()), [])


class TestAdd_abi(AATest):
    def AASetup(self):
        self.pl = ProfileList()

    def testAdd_abi_1(self):
        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule('abi/4.19', False, True))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['abi <abi/4.19>,', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['abi <abi/4.19>,', ''])

    def testAdd_abi_2(self):
        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule('abi/4.19', False, True))
        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule('foo', False, False))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['abi <abi/4.19>,', 'abi "foo",', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['abi <abi/4.19>,', 'abi "foo",', ''])

    def testAdd_abi_error_1(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_abi('/etc/apparmor.d/bin.foo', 'abi/4.19')  # str instead of AbiRule
        self.assertEqual(list(self.pl.files.keys()), [])

    def test_dedup_abi_1(self):
        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule.create_instance('abi <abi/4.19>,'))
        self.pl.add_abi('/etc/apparmor.d/bin.foo', AbiRule.create_instance('   abi     <abi/4.19>  ,  # comment'))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        deleted = self.pl.delete_preamble_duplicates('/etc/apparmor.d/bin.foo')
        self.assertEqual(deleted, 1)
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['abi <abi/4.19>,', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['abi <abi/4.19>,', ''])


class TestAdd_alias(AATest):
    def AASetup(self):
        self.pl = ProfileList()

    def testAdd_alias_1(self):
        self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/foo', '/bar'))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['alias /foo -> /bar,', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['alias /foo -> /bar,', ''])

    def testAdd_alias_2(self):
        self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/foo', '/bar'))
        self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/xyz', '/zyx'))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['alias /foo -> /bar,', 'alias /xyz -> /zyx,', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['alias /foo -> /bar,', 'alias /xyz -> /zyx,', ''])

    def testAdd_alias_two_targets(self):
        self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/foo', '/bar'))
        self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/foo', '/another_target'))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['alias /foo -> /bar,', 'alias /foo -> /another_target,', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['alias /foo -> /bar,', 'alias /foo -> /another_target,', ''])

    def testAdd_alias_error_1(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule(None, '/foo'))  # alias None instead of str
        self.assertEqual(list(self.pl.files.keys()), [])

    def testAdd_alias_error_2(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/foo', None))  # target None instead of str
        self.assertEqual(list(self.pl.files.keys()), [])

    def testAdd_alias_error_3(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_alias('/etc/apparmor.d/bin.foo', 'alias /foo -> /bar,')  # str instead of AliasRule
        self.assertEqual(list(self.pl.files.keys()), [])

    def test_dedup_alias_1(self):
        self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/foo', '/bar'))
        self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/foo', '/another_target'))
        self.pl.add_alias('/etc/apparmor.d/bin.foo', AliasRule('/foo', '/bar'))  # duplicate
        deleted = self.pl.delete_preamble_duplicates('/etc/apparmor.d/bin.foo')
        self.assertEqual(deleted, 1)
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['alias /foo -> /bar,', 'alias /foo -> /another_target,', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['alias /foo -> /bar,', 'alias /foo -> /another_target,', ''])


class TestAdd_variable(AATest):
    def AASetup(self):
        self.pl = ProfileList()

    def testAdd_variable_1(self):
        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule('@{foo}', '=', {'/foo'}))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['@{foo} = /foo', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['@{foo} = /foo', ''])

    def testAdd_variable_2(self):
        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule('@{foo}', '=', {'/foo'}))
        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule('@{bar}', '=', {'/bar'}))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['@{foo} = /foo', '@{bar} = /bar', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['@{foo} = /foo', '@{bar} = /bar', ''])

    def testAdd_variable_error_1(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_variable('/etc/apparmor.d/bin.foo', '@{foo}')  # str instead of IncludeRule
        self.assertEqual(list(self.pl.files.keys()), [])

    def test_dedup_variable_1(self):
        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.create_instance('@{foo} = /foo'))
        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.create_instance('@{foo} += /bar  # comment'))
        self.pl.add_variable('/etc/apparmor.d/bin.foo', VariableRule.create_instance('@{foo}    += /bar /baz'))
        deleted = self.pl.delete_preamble_duplicates('/etc/apparmor.d/bin.foo')
        self.assertEqual(deleted, 1)
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['@{foo} = /foo', '@{foo} += /bar /baz', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['@{foo} = /foo', '@{foo}    += /bar /baz', ''])

    def test_dedup_error_1(self):
        with self.assertRaises(AppArmorBug):
            self.pl.delete_preamble_duplicates('/file/not/found')
        self.assertEqual(list(self.pl.files.keys()), [])


class TestAdd_boolean(AATest):
    def AASetup(self):
        self.pl = ProfileList()

    def testAdd_variable_1(self):
        self.pl.add_boolean('/etc/apparmor.d/bin.foo', BooleanRule('$foo', 'true'))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['$foo = true', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['$foo = true', ''])

    def testAdd_variable_2(self):
        self.pl.add_boolean('/etc/apparmor.d/bin.foo', BooleanRule('$foo', 'true'))
        self.pl.add_boolean('/etc/apparmor.d/bin.foo', BooleanRule('$bar', 'false'))
        self.assertEqual(list(self.pl.files.keys()), ['/etc/apparmor.d/bin.foo'])
        self.assertEqual(self.pl.get_clean('/etc/apparmor.d/bin.foo'), ['$foo = true', '$bar = false', ''])
        self.assertEqual(self.pl.get_raw('/etc/apparmor.d/bin.foo'), ['$foo = true', '$bar = false', ''])

    def testAdd_variable_error_1(self):
        with self.assertRaises(AppArmorBug):
            self.pl.add_boolean('/etc/apparmor.d/bin.foo', '$foo')  # str instead of IncludeRule
        self.assertEqual(list(self.pl.files.keys()), [])


class TestGet(AATest):
    def AASetup(self):
        self.pl = ProfileList()

    def testGet_clean_error(self):
        with self.assertRaises(AppArmorBug):
            self.pl.get_clean('/etc/apparmor.d/not.found')

    def testGet_raw_error(self):
        with self.assertRaises(AppArmorBug):
            self.pl.get_raw('/etc/apparmor.d/not.found')


class AaTest_get_all_merged_variables(AATest):
    tests = ()

    def AASetup(self):
        self.createTmpdir()

        # copy the local profiles to the test directory
        self.profile_dir = self.tmpdir + '/profiles'
        apparmor.aa.profile_dir = self.profile_dir
        shutil.copytree('../../profiles/apparmor.d/', self.profile_dir, symlinks=True)

    def _load_profiles(self):
        apparmor.aa.reset_aa()

        # load the profiles and abstractions
        apparmor.aa.profile_dir = self.profile_dir
        apparmor.aa.loadincludes()
        # Keep this synced with the list in test-logprof.py and
        # test-minitools.py
        apparmor.aa.read_profiles(skip_profiles=skip_active_profiles)

    def test_unchanged(self):
        self._load_profiles()
        prof_filename = os.path.join(self.profile_dir, 'usr.sbin.dnsmasq')
        vars = apparmor.aa.active_profiles.get_all_merged_variables(
            os.path.join(self.profile_dir, 'usr.sbin.dnsmasq'),
            apparmor.aa.include_list_recursive(apparmor.aa.active_profiles.files[prof_filename], True))
        self.assertEqual(vars['@{TFTP_DIR}'], {'/var/tftp', '/srv/tftp', '/srv/tftpboot'})
        self.assertEqual(vars['@{HOME}'], {'@{HOMEDIRS}/*/', '/root/'})

    def test_extended_home(self):
        write_file(self.profile_dir, 'tunables/home.d/extend_home', '@{HOME} += /my/castle/')
        self._load_profiles()
        prof_filename = os.path.join(self.profile_dir, 'usr.sbin.dnsmasq')
        vars = apparmor.aa.active_profiles.get_all_merged_variables(
            os.path.join(self.profile_dir, 'usr.sbin.dnsmasq'),
            apparmor.aa.include_list_recursive(apparmor.aa.active_profiles.files[prof_filename], True))
        self.assertEqual(vars['@{TFTP_DIR}'], {'/var/tftp', '/srv/tftp', '/srv/tftpboot'})
        self.assertEqual(vars['@{HOME}'], {'@{HOMEDIRS}/*/', '/root/', '/my/castle/'})

    def test_extended_home_2(self):
        write_file(self.profile_dir, 'tunables/home.d/extend_home', '@{HOME} += /my/castle/')
        write_file(self.profile_dir, 'tunables/home.d/moving_around', '@{HOME} += /on/the/road/')
        self._load_profiles()
        prof_filename = os.path.join(self.profile_dir, 'usr.sbin.dnsmasq')
        vars = apparmor.aa.active_profiles.get_all_merged_variables(
            os.path.join(self.profile_dir, 'usr.sbin.dnsmasq'),
            apparmor.aa.include_list_recursive(apparmor.aa.active_profiles.files[prof_filename], True))
        self.assertEqual(vars['@{TFTP_DIR}'], {'/var/tftp', '/srv/tftp', '/srv/tftpboot'})
        self.assertEqual(vars['@{HOME}'], {'@{HOMEDIRS}/*/', '/root/', '/my/castle/', '/on/the/road/'})

    def test_extend_home_in_mainfile(self):
        write_file(self.profile_dir, 'tunables/home.d/extend_home', '@{HOME} += /my/castle/')
        write_file(self.profile_dir, 'dummy_profile', 'include <tunables/global>\n@{HOME} += /in/the/profile/')
        self._load_profiles()
        prof_filename = os.path.join(self.profile_dir, 'dummy_profile')
        vars = apparmor.aa.active_profiles.get_all_merged_variables(
            os.path.join(self.profile_dir, 'dummy_profile'),
            apparmor.aa.include_list_recursive(apparmor.aa.active_profiles.files[prof_filename], True))
        self.assertEqual(vars.get('@{TFTP_DIR}', None), None)
        self.assertEqual(vars['@{HOME}'], {'@{HOMEDIRS}/*/', '/root/', '/my/castle/', '/in/the/profile/'})

    def test_redefine_home(self):
        write_file(self.profile_dir, 'tunables/home.d/overwrite_home', '@{HOME} = /my/castle/')  # note: =, not +=
        self._load_profiles()
        prof_filename = os.path.join(self.profile_dir, 'usr.sbin.dnsmasq')
        with self.assertRaises(AppArmorException):
            apparmor.aa.active_profiles.get_all_merged_variables(
                os.path.join(self.profile_dir, 'usr.sbin.dnsmasq'),
                apparmor.aa.include_list_recursive(apparmor.aa.active_profiles.files[prof_filename], True))

    def test_add_to_nonexisting(self):
        write_file(self.profile_dir, 'tunables/home.d/no_such_var', '@{NO_SUCH_HOME} += /my/castle/')  # add to non-existing variable
        self._load_profiles()
        prof_filename = os.path.join(self.profile_dir, 'usr.sbin.dnsmasq')
        with self.assertRaises(AppArmorException):
            apparmor.aa.active_profiles.get_all_merged_variables(
                os.path.join(self.profile_dir, 'usr.sbin.dnsmasq'),
                apparmor.aa.include_list_recursive(apparmor.aa.active_profiles.files[prof_filename], True))

    def test_vars_from_nonexisting_profile(self):
        with self.assertRaises(AppArmorBug):
            apparmor.aa.active_profiles.get_all_merged_variables(
                os.path.join(self.profile_dir, 'file.not.found'), list())


class TestGet_profile_and_childs(AATest):
    def AASetup(self):
        self.pl = ProfileList()
        self.dummy_profile = ProfileStorage('TEST DUMMY', 'AATest_no_file', 'TEST')

    def testGet_profile_and_childs1(self):
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'bafoo',    '/bin/bafoo',    self.dummy_profile)
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo',      '/bin/foo',      self.dummy_profile)
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foobar',   '/bin/foobar',   self.dummy_profile)
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo//bar', '/bin/foo//bar', self.dummy_profile)
        self.pl.add_profile('/etc/apparmor.d/bin.foo', 'foo//xy',  '/bin/foo//xy',  self.dummy_profile)

        expected = ['foo', 'foo//bar', 'foo//xy']

        self.assertEqual(list(self.pl.get_profile_and_childs('foo')), expected)


setup_aa(apparmor.aa)
setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



# ----------------------------------------------------------------------
#    Copyright (C) 2013 Kshitij Gupta <kgupta8592@gmail.com>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------
import unittest

import apparmor.config as config


class Test(unittest.TestCase):

    def test_IniConfig(self):
        ini_config = config.Config('ini')
        ini_config.CONF_DIR = '.'
        conf = ini_config.read_config('logprof.conf')
        logprof_sections = ['settings', 'qualifiers', 'required_hats', 'defaulthat', 'globs']
        logprof_sections_options = [
            'profiledir', 'inactive_profiledir', 'logfiles', 'parser',
            'logger', 'default_owner_prompt', 'custom_includes']
        logprof_settings_parser = '../../parser/apparmor_parser ../parser/apparmor_parser'

        self.assertEqual(conf.sections(), logprof_sections)
        self.assertEqual(conf.options('settings'), logprof_sections_options)
        self.assertEqual(conf['settings']['parser'], logprof_settings_parser)

    def test_ShellConfig(self):
        shell_config = config.Config('shell')
        shell_config.CONF_DIR = '.'
        conf = shell_config.read_config('easyprof.conf')
        easyprof_sections = ['POLICYGROUPS_DIR', 'TEMPLATES_DIR']
        easyprof_policygroup = './policygroups'
        easyprof_templates = './templates'

        self.assertEqual(sorted(conf[''].keys()), easyprof_sections)
        self.assertEqual(conf['']['POLICYGROUPS_DIR'], easyprof_policygroup)
        self.assertEqual(conf['']['TEMPLATES_DIR'], easyprof_templates)


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testConfig']
    unittest.main()



# ----------------------------------------------------------------------
#    Copyright (C) 2013 Kshitij Gupta <kgupta8592@gmail.com>
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------
import inspect
import os
import shutil
import sys
import tempfile
import unittest


# def test_readkey(self):
#     print("Please press the Y button on the keyboard.")
#     self.assertEqual(apparmor.common.readkey().lower(), 'y', 'Error reading key from shell!')

# If a profile can't be parsed by the tools, add it to skip_active_profiles or skip_extra_profiles.
# Add only the filename (without path), for example 'usr.bin.foo'.
# These skip lists are meant as a temporary solution, and should be empty on release.
skip_active_profiles = []

skip_extra_profiles = []


class AATest(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        self.AASetup()

    def AASetup(self):
        """override this function if a test needs additional setup steps (instead of overriding setUp())"""

    def tearDown(self):
        if self.tmpdir and os.path.exists(self.tmpdir):
            shutil.rmtree(self.tmpdir)

        self.AATeardown()

    def AATeardown(self):
        """override this function if a test needs additional teardown steps (instead of overriding tearDown())"""

    def createTmpdir(self):
        self.tmpdir = tempfile.mkdtemp(prefix='aa-test-')

    def writeTmpfile(self, file, contents):
        if not self.tmpdir:
            self.createTmpdir()
        return write_file(self.tmpdir, file, contents)

    tests = ()
    tmpdir = None


class AAParseTest(unittest.TestCase):
    parse_function = None

    def _test_parse_rule(self, rule):
        self.assertIsNot(self.parse_function, 'Test class did not set a parse_function')
        parsed = self.parse_function(rule)
        self.assertEqual(
            rule, parsed.serialize(),
            'parse object {} returned "{}", expected "{}"'.format(
                self.parse_function.__doc__, parsed.serialize(), rule))


def setup_all_loops(module_name):
    """call setup_tests_loop() for each class in module_name"""
    for name, obj in inspect.getmembers(sys.modules[module_name]):
        if inspect.isclass(obj):
            if issubclass(obj, unittest.TestCase):
                setup_tests_loop(obj)


def setup_tests_loop(test_class):
    """Create tests in test_class using test_class.tests and self._run_test()

    test_class.tests should be tuples of (test_data, expected_results)
    test_data and expected_results can be of any type as long as test_class._run_test()
    know how to handle them.

    A typical definition for _run_test() is:
        def test_class._run_test(self, test_data, expected)
    """

    for (i, (test_data, expected)) in enumerate(test_class.tests):
        def stub_test(self, test_data=test_data, expected=expected):
            self._run_test(test_data, expected)

        stub_test.__doc__ = "test '{}'".format(test_data)
        setattr(test_class, 'test_{}'.format(i), stub_test)


def setup_regex_tests(test_class):
    """Create tests in test_class using test_class.tests and AAParseTest._test_parse_rule()

    test_class.tests should be tuples of (line, description)
    """
    for (i, (line, desc)) in enumerate(test_class.tests):
        def stub_test(self, line=line):
            self._test_parse_rule(line)

        stub_test.__doc__ = "test '{}': {}".format(line, desc)
        setattr(test_class, 'test_{}'.format(i), stub_test)


def setup_aa(aa):
    confdir = os.getenv('__AA_CONFDIR')
    try:
        if confdir:
            aa.init_aa(confdir=confdir)
        else:
            aa.init_aa()
    except AttributeError:
        # apparmor.aa module versions <= 2.11 do not have the init_aa() method
        pass


def write_file(directory, file, contents):
    """construct path, write contents to it, and return the constructed path"""
    path = os.path.join(directory, file)
    with open(path, 'w+') as f:
        f.write(contents)
    return path


def read_file(path):
    """read and return file contents"""
    with open(path, 'r') as f:
        return f.read()


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.test_RegexParser']
    unittest.main()



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2022 Canonical, Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple
from common_test import AATest, setup_all_loops

from apparmor.rule.mqueue import MessageQueueRule, MessageQueueRuleset
from apparmor.common import AppArmorException, AppArmorBug
from apparmor.translations import init_translation
_ = init_translation()


class MessageQueueTestParse(AATest):
    tests = (
        #                                                            access                         type                    label                   mqueue_name             audit   deny    allow   comment
        ('mqueue,',                                 MessageQueueRule(MessageQueueRule.ALL,          MessageQueueRule.ALL,   MessageQueueRule.ALL,   MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue create,',                          MessageQueueRule(('create'),                    MessageQueueRule.ALL,   MessageQueueRule.ALL,   MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue (create,open,delete),',            MessageQueueRule(('create', 'open', 'delete'),  MessageQueueRule.ALL,   MessageQueueRule.ALL,   MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue (getattr,setattr),',               MessageQueueRule(('getattr', 'setattr'),        MessageQueueRule.ALL,   MessageQueueRule.ALL,   MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue (write,read),',                    MessageQueueRule(('write', 'read'),             MessageQueueRule.ALL,   MessageQueueRule.ALL,   MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue (open,delete),',                   MessageQueueRule(('open', 'delete'),            MessageQueueRule.ALL,   MessageQueueRule.ALL,   MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue write label=foo,',                 MessageQueueRule(('write'),                     MessageQueueRule.ALL,   'foo',                  MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue read label=foo /queue,',           MessageQueueRule(('read'),                      MessageQueueRule.ALL,   'foo',                  '/queue',               False,  False,  False,  '')),
        ('audit mqueue read label=foo /queue,',     MessageQueueRule(('read'),                      MessageQueueRule.ALL,   'foo',                  '/queue',               True,   False,  False,  '')),
        ('deny mqueue rw label=foo /queue,',        MessageQueueRule(('rw'),                        MessageQueueRule.ALL,   'foo',                  '/queue',               False,  True,   False,  '')),
        ('audit allow mqueue r label=foo /queue,',  MessageQueueRule(('r'),                         MessageQueueRule.ALL,   'foo',                  '/queue',               True,   False,  True,   '')),
        ('mqueue w label=foo 1234, # cmt',          MessageQueueRule(('w'),                         MessageQueueRule.ALL,   'foo',                  '1234',                 False,  False,  False,  ' # cmt')),
        ('mqueue wr 1234,',                         MessageQueueRule(('wr'),                        MessageQueueRule.ALL,   MessageQueueRule.ALL,   '1234',                 False,  False,  False,  '')),
        ('mqueue 1234,',                            MessageQueueRule(MessageQueueRule.ALL,          MessageQueueRule.ALL,   MessageQueueRule.ALL,   '1234',                 False,  False,  False,  '')),
        ('mqueue type=sysv,',                       MessageQueueRule(MessageQueueRule.ALL,          'sysv',                 MessageQueueRule.ALL,   MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue type=posix,',                      MessageQueueRule(MessageQueueRule.ALL,          'posix',                MessageQueueRule.ALL,   MessageQueueRule.ALL,   False,  False,  False,  '')),
        ('mqueue type=sysv 1234,',                  MessageQueueRule(MessageQueueRule.ALL,          'sysv',                 MessageQueueRule.ALL,   '1234',                 False,  False,  False,  '')),
        ('mqueue type=posix /queue,',               MessageQueueRule(MessageQueueRule.ALL,          'posix',                MessageQueueRule.ALL,   '/queue',               False,  False,  False,  '')),
        ('mqueue open type=sysv label=foo 1234,',   MessageQueueRule(('open'),                      'sysv',                 'foo',                  '1234',                 False,  False,  False,  '')),
        ('mqueue  r type=posix /,',                 MessageQueueRule(('r'),                         'posix',                MessageQueueRule.ALL,   '/',                    False,  False,  False,  '')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(MessageQueueRule.match(rawrule))
        obj = MessageQueueRule.create_instance(rawrule)
        expected.raw_rule = rawrule.strip()
        self.assertTrue(obj.is_equal(expected, True))


class MessageQueueTestParseInvalid(AATest):
    tests = (
        ('mqueue label=,',                   AppArmorException),
        ('mqueue invalidaccess /queuename,', AppArmorException),
        ('mqueue invalidqueuename,',         AppArmorException),
        ('mqueue invalidqueuename1234,',     AppArmorException),
        ('mqueue foo label foo bar,',        AppArmorException),
        ('mqueue type=,',                    AppArmorException),
        ('mqueue type=sysv /foo,',           AppArmorException),
        ('mqueue type=posix 1234,',          AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(MessageQueueRule.match(rawrule))  # the above invalid rules still match the main regex!
        with self.assertRaises(expected):
            MessageQueueRule.create_instance(rawrule)

    def test_parse_fail(self):
        with self.assertRaises(AppArmorException):
            MessageQueueRule.create_instance('foo,')

    def test_diff_non_mqueuerule(self):
        exp = namedtuple('exp', ('audit', 'deny', 'priority'))
        obj = MessageQueueRule(('open'), 'posix', 'bar', '/foo')
        with self.assertRaises(AppArmorBug):
            obj.is_equal(exp(False, False, None), False)

    def test_diff_access(self):
        obj1 = MessageQueueRule(('open'), 'posix', 'bar', '/foo')
        obj2 = MessageQueueRule(('create'), 'posix', 'bar', '/foo')
        self.assertFalse(obj1.is_equal(obj2, False))

    def test_diff_type(self):
        obj1 = MessageQueueRule(('open'), 'sysv', 'bar', MessageQueueRule.ALL)
        obj2 = MessageQueueRule(('open'), 'posix', 'inv', MessageQueueRule.ALL)
        self.assertFalse(obj1.is_equal(obj2, False))

    def test_diff_label(self):
        obj1 = MessageQueueRule(('open'), 'posix', 'bar', '/foo')
        obj2 = MessageQueueRule(('open'), 'posix', 'inv', '/foo')
        self.assertFalse(obj1.is_equal(obj2, False))

    def test_diff_mqueue_name(self):
        obj1 = MessageQueueRule(('open'), MessageQueueRule.ALL, 'bar', '/foo')
        obj2 = MessageQueueRule(('open'), MessageQueueRule.ALL, 'bar', '123')
        self.assertFalse(obj1.is_equal(obj2, False))


class InvalidMessageQueueInit(AATest):
    tests = (
        # init params                               expected exception
        (('write', 'sysv', '', '/foo'), AppArmorBug),  # empty label
        (('write', '', 'bar', '/foo'), AppArmorBug),  # empty type
        (('', 'sysv', 'bar', '/foo'), AppArmorBug),  # empty access
        (('write', 'sysv', 'bar', ''), AppArmorBug),  # empty mqueue_name
        (('    ', 'sysv', 'bar', '/foo'), AppArmorBug),  # whitespace access
        (('write', '    ', 'bar', '/foo'), AppArmorBug),  # whitespace type
        (('write', 'sysv', '   ', '/foo'), AppArmorBug),  # whitespace label
        (('write', 'sysv', 'bar', '    '), AppArmorBug),  # whitespace mqueue_name
        (('xyxy', 'sysv', 'bar', '/foo'), AppArmorException),  # invalid access
        ((dict(), '', 'bar', '/foo'), AppArmorBug),  # wrong type for access
        ((None, '', 'bar', '/foo'), AppArmorBug),  # wrong type for access
        (('write', dict(), 'bar', '/foo'), AppArmorBug),  # wrong type for type
        (('write', None, 'bar', '/foo'), AppArmorBug),  # wrong type for type
        (('write', '', dict(), '/foo'), AppArmorBug),  # wrong type for label
        (('write', '', None, '/foo'), AppArmorBug),  # wrong type for label
        (('write', '', 'bar', dict()), AppArmorBug),  # wrong type for mqueue_name
        (('write', '', 'bar', None), AppArmorBug),  # wrong type for mqueue_name
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            MessageQueueRule(*params)

    def test_missing_params_1(self):
        with self.assertRaises(TypeError):
            MessageQueueRule()

    def test_missing_params_2(self):
        with self.assertRaises(TypeError):
            MessageQueueRule('r')

    def test_missing_params_3(self):
        with self.assertRaises(TypeError):
            MessageQueueRule('r', 'sysv')

    def test_missing_params_4(self):
        with self.assertRaises(TypeError):
            MessageQueueRule('r', 'sysv', 'foo')


class WriteMessageQueueTestAATest(AATest):
    tests = (
        #  raw rule                                               clean rule
        ('     mqueue         ,    # foo        ',                'mqueue, # foo'),
        ('    audit     mqueue create,',                          'audit mqueue create,'),
        ('    audit     mqueue (open  ),',                        'audit mqueue open,'),
        ('    audit     mqueue (delete , read ),',                'audit mqueue (delete read),'),
        ('   deny mqueue         write      label=bar,# foo bar', 'deny mqueue write label=bar, # foo bar'),
        ('   deny mqueue         open      ,# foo bar',           'deny mqueue open, # foo bar'),
        ('   allow mqueue             label=tst    ,# foo bar',   'allow mqueue label=tst, # foo bar'),
        ('mqueue,',                                               'mqueue,'),
        ('mqueue (read),',                                        'mqueue read,'),
        ('mqueue (create),',                                      'mqueue create,'),
        ('mqueue (write read),',                                  'mqueue (read write),'),
        ('mqueue (open,create,open,delete,write,read),',          'mqueue (create delete open read write),'),
        ('mqueue r,',                                             'mqueue r,'),
        ('mqueue w,',                                             'mqueue w,'),
        ('mqueue rw,',                                            'mqueue rw,'),
        ('mqueue delete label="tst",',                            'mqueue delete label="tst",'),
        ('mqueue (getattr) label=bar,',                           'mqueue getattr label=bar,'),
        ('mqueue getattr /foo,',                                  'mqueue getattr /foo,'),
        ('mqueue (setattr getattr) 1234,',                        'mqueue (getattr setattr) 1234,'),
        ('mqueue wr label=tst 1234,',                             'mqueue wr label=tst 1234,'),
        ('mqueue wr  type=sysv   label=tst 1234,',                'mqueue wr type=sysv label=tst 1234,'),
        ('mqueue wr   type=posix label=tst /foo,',                'mqueue wr type=posix label=tst /foo,'),
        ('  priority = -82 mqueue getattr /foo,',                 'priority=-82 mqueue getattr /foo,'),
        ('  priority =  12 audit mqueue (setattr getattr) 1234,', 'priority=12 audit mqueue (getattr setattr) 1234,'),
        ('  priority=0 mqueue getattr /foo,',                     'priority=0 mqueue getattr /foo,'),
        ('  priority=+82 mqueue getattr /foo,',                   'priority=82 mqueue getattr /foo,'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(MessageQueueRule.match(rawrule))
        obj = MessageQueueRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    def test_write_manually(self):
        obj = MessageQueueRule('setattr', 'posix', 'bar', '/foo', allow_keyword=True)

        expected = '    allow mqueue setattr type=posix label=bar /foo,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')

    def test_write_invalid_access(self):
        obj = MessageQueueRule('setattr', 'posix', 'bar', '/foo')
        obj.access = ''
        with self.assertRaises(AppArmorBug):
            obj.get_clean()

    def test_write_invalid_type(self):
        obj = MessageQueueRule('setattr', 'posix', 'bar', '/foo')
        obj.mqueue_type = ''
        with self.assertRaises(AppArmorBug):
            obj.get_clean()

    def test_write_invalid_label(self):
        obj = MessageQueueRule('setattr', 'posix', 'bar', '/foo')
        obj.label = ''
        with self.assertRaises(AppArmorBug):
            obj.get_clean()

    def test_write_invalid_mqueue_name(self):
        obj = MessageQueueRule('setattr', 'posix', 'bar', '/foo')
        obj.mqueue_name = ''
        with self.assertRaises(AppArmorBug):
            obj.get_clean()


class MessageQueueIsCoveredTest(AATest):
    def test_is_covered(self):
        obj = MessageQueueRule(('create'), MessageQueueRule.ALL, 'f*', MessageQueueRule.ALL)
        self.assertTrue(obj.is_covered(MessageQueueRule(('create'), 'sysv', 'f*', '1234')))
        self.assertTrue(obj.is_covered(MessageQueueRule(('create'), 'posix', 'f*', MessageQueueRule.ALL)))
        self.assertTrue(obj.is_covered(MessageQueueRule(('create'), 'sysv', 'foo', MessageQueueRule.ALL)))
        self.assertTrue(obj.is_covered(MessageQueueRule(('create'), 'sysv', 'foo', '1234')))

    def test_is_not_covered(self):
        obj = MessageQueueRule(('getattr'), 'sysv', 'f*', '1234')
        self.assertFalse(obj.is_covered(MessageQueueRule(('create'), 'sysv', 'foo', MessageQueueRule.ALL)))
        self.assertFalse(obj.is_covered(MessageQueueRule(('getattr'), 'posix', 'foo', MessageQueueRule.ALL)))
        self.assertFalse(obj.is_covered(MessageQueueRule(('getattr'), 'sysv', 'bar', MessageQueueRule.ALL)))
        self.assertFalse(obj.is_covered(MessageQueueRule(('getattr'), 'sysv', 'foo', '123')))


class MessageQueueLogprofHeaderTest(AATest):
    tests = (
        ('mqueue,',                     [                               _('Access mode'), _('ALL'),         _('Type'), _('ALL'), _('Label'), _('ALL'), _('Message queue name'), _('ALL'), ]),  # noqa: E201
        ('mqueue (create,getattr) 12,', [                               _('Access mode'), 'create getattr', _('Type'), _('ALL'), _('Label'), _('ALL'), _('Message queue name'), '12', ]),  # noqa: E201
        ('mqueue write label=bar,',     [                               _('Access mode'), 'write',          _('Type'), _('ALL'), _('Label'), 'bar',    _('Message queue name'), _('ALL'), ]),  # noqa: E201
        ('mqueue write type=sysv,',     [                               _('Access mode'), 'write',          _('Type'), 'sysv',   _('Label'), _('ALL'), _('Message queue name'), _('ALL'), ]),  # noqa: E201
        ('mqueue read type=posix,',     [                               _('Access mode'), 'read',           _('Type'), 'posix',  _('Label'), _('ALL'), _('Message queue name'), _('ALL'), ]),  # noqa: E201
        ('deny mqueue read /foo,',      [_('Qualifier'), 'deny',        _('Access mode'), 'read',           _('Type'), _('ALL'), _('Label'), _('ALL'), _('Message queue name'), '/foo', ]),
        ('allow mqueue setattr,',       [_('Qualifier'), 'allow',       _('Access mode'), 'setattr',        _('Type'), _('ALL'), _('Label'), _('ALL'), _('Message queue name'), _('ALL'), ]),
        ('audit mqueue r label=ba 12,', [_('Qualifier'), 'audit',       _('Access mode'), 'r',              _('Type'), _('ALL'), _('Label'), 'ba',     _('Message queue name'), '12', ]),
        ('audit deny mqueue rw,',       [_('Qualifier'), 'audit deny',  _('Access mode'), 'rw',             _('Type'), _('ALL'), _('Label'), _('ALL'), _('Message queue name'), _('ALL'), ]),
    )

    def _run_test(self, params, expected):
        obj = MessageQueueRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


class MessageQueueGlobTestAATest(AATest):
    def test_glob(self):
        self.assertEqual(MessageQueueRuleset().get_glob('mqueue create,'), 'mqueue,')


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2023 Canonical, Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple
from common_test import AATest, setup_all_loops

from apparmor.rule.io_uring import IOUringRule, IOUringRuleset
from apparmor.common import AppArmorException, AppArmorBug
from apparmor.translations import init_translation
_ = init_translation()


class IOUringTestParse(AATest):
    tests = (
        #                                                             access                        label            audit  deny   allow  comment
        ('io_uring,',                                     IOUringRule(IOUringRule.ALL,              IOUringRule.ALL, False, False, False, '')),
        ('io_uring sqpoll,',                              IOUringRule(('sqpoll'),                   IOUringRule.ALL, False, False, False, '')),
        ('io_uring override_creds,',                      IOUringRule(('override_creds'),           IOUringRule.ALL, False, False, False, '')),
        ('io_uring override_creds label=/foo,',           IOUringRule(('override_creds'),           '/foo',          False, False, False, '')),
        ('io_uring sqpoll label=bar,',                    IOUringRule(('sqpoll'),                    'bar',           False, False, False, '')),
        ('io_uring (override_creds, sqpoll) label=/foo,', IOUringRule(('override_creds', 'sqpoll'), '/foo',          False, False, False, '')),
        ('audit io_uring sqpoll,',                        IOUringRule(('sqpoll'),                   IOUringRule.ALL, True,  False, False, '')),
        ('deny io_uring,',                                IOUringRule(IOUringRule.ALL,              IOUringRule.ALL, False, True,  False, '')),
        ('deny io_uring (sqpoll, override_creds),',       IOUringRule(('sqpoll', 'override_creds'), IOUringRule.ALL, False, True,  False, '')),
        ('audit allow io_uring,',                         IOUringRule(IOUringRule.ALL,              IOUringRule.ALL, True,  False, True,  '')),
        ('io_uring override_creds, # cmt',                IOUringRule(('override_creds'),           IOUringRule.ALL, False, False, False, ' # cmt')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(IOUringRule.match(rawrule))
        obj = IOUringRule.create_instance(rawrule)
        expected.raw_rule = rawrule.strip()
        self.assertTrue(obj.is_equal(expected, True))


class IOUringTestParseInvalid(AATest):
    tests = (
        ('io_uring invalidaccess,',           AppArmorException),
        ('io_uring label=,',                  AppArmorException),
        ('io_uring invalidaccess label=foo,', AppArmorException),
        ('io_uring sqpoll label=,',           AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(IOUringRule.match(rawrule))  # the above invalid rules still match the main regex!
        with self.assertRaises(expected):
            IOUringRule.create_instance(rawrule)

    def test_parse_fail(self):
        with self.assertRaises(AppArmorException):
            IOUringRule.create_instance('foo,')

    def test_diff_non_iouringrule(self):
        exp = namedtuple('exp', ('audit', 'deny', 'priority'))
        obj = IOUringRule(('sqpoll'), IOUringRule.ALL)
        with self.assertRaises(AppArmorBug):
            obj.is_equal(exp(False, False, None), False)

    def test_diff_access(self):
        obj1 = IOUringRule(IOUringRule.ALL, IOUringRule.ALL)
        obj2 = IOUringRule(('sqpoll'), IOUringRule.ALL)
        self.assertFalse(obj1.is_equal(obj2, False))

    def test_diff_label(self):
        obj1 = IOUringRule(IOUringRule.ALL, 'foo')
        obj2 = IOUringRule(IOUringRule.ALL, '/bar')
        self.assertFalse(obj1.is_equal(obj2, False))


class InvalidIOUringInit(AATest):
    tests = (
        # access    label      expected exception
        (('',       'label'),  AppArmorBug),        # empty access
        (('    ',   'label'),  AppArmorBug),        # whitespace access
        (('xyxy',   'label'),  AppArmorException),  # invalid access
        ((dict(),   'label'),  AppArmorBug),        # wrong type for access
        ((None,     'label'),  AppArmorBug),        # wrong type for access
        (('sqpoll', ''),       AppArmorBug),        # empty label
        (('sqpoll', '    '),   AppArmorBug),        # whitespace label
        (('sqpoll', dict()),   AppArmorBug),        # wrong type for label
        (('sqpoll', None),     AppArmorBug),        # wrong type for label
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            IOUringRule(*params)

    def test_missing_params1(self):
        with self.assertRaises(TypeError):
            IOUringRule()

    def test_missing_params2(self):
        with self.assertRaises(TypeError):
            IOUringRule('override_creds')


class WriteIOUringTestAATest(AATest):
    tests = (
        #  raw rule                                                  clean rule
        ('     io_uring         ,    # foo        ',                 'io_uring, # foo'),
        ('    audit     io_uring sqpoll,',                           'audit io_uring sqpoll,'),
        ('    audit     io_uring (override_creds  ),',               'audit io_uring override_creds,'),
        ('    audit     io_uring (sqpoll , override_creds ),',       'audit io_uring (override_creds sqpoll),'),
        ('   deny io_uring         sqpoll      label=bar,# foo bar', 'deny io_uring sqpoll label=bar, # foo bar'),
        ('   deny io_uring         override_creds      ,# foo bar',  'deny io_uring override_creds, # foo bar'),
        ('   allow io_uring             label=tst    ,# foo bar',    'allow io_uring label=tst, # foo bar'),
        ('io_uring,',                                                'io_uring,'),
        ('io_uring (override_creds),',                               'io_uring override_creds,'),
        ('io_uring (sqpoll),',                                       'io_uring sqpoll,'),
        ('io_uring (sqpoll override_creds),',                        'io_uring (override_creds sqpoll),'),
        ('io_uring sqpoll label="tst",',                             'io_uring sqpoll label="tst",'),
        ('io_uring (override_creds) label=bar,',                     'io_uring override_creds label=bar,'),
        ('io_uring (sqpoll override_creds) label=/foo,',             'io_uring (override_creds sqpoll) label=/foo,'),
        (' priority=1   deny io_uring  override_creds  ,# foo bar',  'priority=1 deny io_uring override_creds, # foo bar'),
        (' priority=0   deny io_uring  override_creds  ,# foo bar',  'priority=0 deny io_uring override_creds, # foo bar'),
        (' priority=-23 deny io_uring  override_creds  ,# foo bar',  'priority=-23 deny io_uring override_creds, # foo bar'),
        (' priority=+21 deny io_uring  override_creds  ,# foo bar',  'priority=21 deny io_uring override_creds, # foo bar'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(IOUringRule.match(rawrule))
        obj = IOUringRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    def test_write_manually(self):
        obj = IOUringRule('sqpoll', IOUringRule.ALL, allow_keyword=True)

        expected = '    allow io_uring sqpoll,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')

    def test_write_invalid_access(self):
        obj = IOUringRule('sqpoll', IOUringRule.ALL)
        obj.access = ''
        with self.assertRaises(AppArmorBug):
            obj.get_clean()

    def test_write_invalid_label(self):
        obj = IOUringRule(IOUringRule.ALL, 'bar')
        obj.label = ''
        with self.assertRaises(AppArmorBug):
            obj.get_clean()


class IOUringIsCoveredTest(AATest):
    def test_is_covered(self):
        obj = IOUringRule(IOUringRule.ALL, 'ba*')
        self.assertTrue(obj.is_covered(IOUringRule(('sqpoll'), 'ba')))
        self.assertTrue(obj.is_covered(IOUringRule(IOUringRule.ALL, 'baz')))

    def test_is_not_covered(self):
        obj = IOUringRule(('sqpoll'), 'foo')
        self.assertFalse(obj.is_covered(IOUringRule(IOUringRule.ALL, 'foo')))
        self.assertFalse(obj.is_covered(IOUringRule(('sqpoll'), IOUringRule.ALL)))

    def test_is_covered_priority(self):
        obj = IOUringRule(IOUringRule.ALL, 'ba*', priority=0)
        prio_obj = IOUringRule(IOUringRule.ALL, 'ba*', priority=1)
        self.assertFalse(obj.is_covered(prio_obj))
        self.assertTrue(prio_obj.is_covered(obj))

    def test_is_covered_priority_2(self):
        obj = IOUringRule(IOUringRule.ALL, 'ba*')
        obj2 = IOUringRule(IOUringRule.ALL, 'ba*', priority=0)
        self.assertTrue(obj.is_covered(obj2))
        self.assertTrue(obj2.is_covered(obj))
        self.assertTrue(obj.is_equal(obj2))
        self.assertTrue(obj2.is_equal(obj))
        self.assertFalse(obj.is_equal(obj2, strict=True))
        self.assertFalse(obj2.is_equal(obj, strict=True))


class IOUringLogprofHeaderTest(AATest):
    tests = (
        ('io_uring,',        [_('Access mode'), _('ALL'), _('Label'), _('ALL')]),
        ('io_uring sqpoll,', [_('Access mode'), 'sqpoll', _('Label'), _('ALL')]),
        ('io_uring override_creds,', [_('Access mode'), 'override_creds', _('Label'), _('ALL')]),
        ('io_uring (sqpoll,override_creds),', [_('Access mode'), 'override_creds sqpoll', _('Label'), _('ALL')]),
        ('io_uring sqpoll label=/foo,', [_('Access mode'), 'sqpoll', _('Label'), '/foo']),
        ('io_uring override_creds label=bar,', [_('Access mode'), 'override_creds', _('Label'), 'bar']),
        ('io_uring (sqpoll,override_creds) label=baz,', [_('Access mode'), 'override_creds sqpoll', _('Label'), 'baz']),
    )

    def _run_test(self, params, expected):
        obj = IOUringRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


class IOUringGlobTestAATest(AATest):
    def test_glob(self):
        self.assertEqual(IOUringRuleset().get_glob('io_uring sqpoll,'), 'io_uring,')


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple

from apparmor.common import AppArmorBug, AppArmorException
from apparmor.logparser import ReadLog
from apparmor.aare import AARE
from apparmor.rule.pivot_root import PivotRootRule, PivotRootRuleset
from apparmor.translations import init_translation
from common_test import AATest, setup_all_loops

_ = init_translation()

exp = namedtuple(
    'exp', ('audit', 'allow_keyword', 'deny', 'comment', 'oldroot', 'all_oldroots', 'newroot',
            'all_newroots', 'profile_name', 'all_profile_names'),
)


# # --- tests for single PivotRootRule --- #

class PivotRootTest(AATest):
    def _compare_obj(self, obj, expected):
        self.assertEqual(expected.audit, obj.audit)
        self.assertEqual(expected.allow_keyword, obj.allow_keyword)
        self.assertEqual(expected.deny, obj.deny)
        self.assertEqual(expected.comment, obj.comment)

        if type(obj.oldroot) is AARE:
            self.assertEqual(expected.oldroot, obj.oldroot.regex)
        else:
            self.assertEqual(expected.oldroot, obj.oldroot)

        self.assertEqual(expected.all_oldroots, obj.all_oldroots)

        if type(obj.newroot) is AARE:
            self.assertEqual(expected.newroot, obj.newroot.regex)
        else:
            self.assertEqual(expected.newroot, obj.newroot)

        self.assertEqual(expected.all_newroots, obj.all_newroots)

        if type(obj.profile_name) is AARE:
            self.assertEqual(expected.profile_name, obj.profile_name.regex)
        else:
            self.assertEqual(expected.profile_name, obj.profile_name)

        self.assertEqual(expected.all_profile_names, obj.all_profile_names)


class PivotRootTestParse(PivotRootTest):
    tests = (
        # PivotRootRule object                                              audit  allow  deny   comment   oldroot      all?    newroot         all?    profile_name    all?
        ('pivot_root,',                                                 exp(False, False, False, '',       None,        True,   None,           True,   None,           True)),
        ('pivot_root oldroot=/oldroot,                         # cmt',  exp(False, False, False, ' # cmt', '/oldroot',  False,  None,           True,   None,           True)),
        ('pivot_root oldroot=/oldroot /new/root,               # cmt',  exp(False, False, False, ' # cmt', '/oldroot',  False,  '/new/root',    False,  None,           True)),
        ('pivot_root oldroot=/oldroot /new/root -> targetprof, # cmt',  exp(False, False, False, ' # cmt', '/oldroot',  False,  '/new/root',    False,  'targetprof',   False)),
        ('pivot_root oldroot=/oldroot           -> targetprof, # cmt',  exp(False, False, False, ' # cmt', '/oldroot',  False,  None,           True,   'targetprof',   False)),
        ('pivot_root                  /new/root,               # cmt',  exp(False, False, False, ' # cmt', None,        True,   '/new/root',    False,  None,           True)),
        ('pivot_root                  /new/root -> targetprof, # cmt',  exp(False, False, False, ' # cmt', None,        True,   '/new/root',    False,  'targetprof',   False)),
        ('pivot_root                            -> targetprof, # cmt',  exp(False, False, False, ' # cmt', None,        True,   None,           True,   'targetprof',   False)),
        ('pivot_root oldroot="/oldroot",                       # cmt',  exp(False, False, False, ' # cmt', '/oldroot',  False,  None,           True,   None,           True)),
        ('pivot_root                  "/new/root",             # cmt',  exp(False, False, False, ' # cmt', None,        True,   '/new/root',    False,  None,           True)),
        ('pivot_root                          -> "targetprof", # cmt',  exp(False, False, False, ' # cmt', None,        True,   None,           True,   'targetprof',   False)),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(PivotRootRule.match(rawrule))
        obj = PivotRootRule.create_instance(rawrule)
        self.assertEqual(rawrule.strip(), obj.raw_rule)
        self._compare_obj(obj, expected)


class PivotRootTestParseInvalid(PivotRootTest):
    tests = (
        ('pivot_root foo,',         AppArmorException),
        ('pivot_root foo bar,',     AppArmorException),
        ('pivot_root oldroot= ,',   AppArmorException),
        ('pivot_root ->  ,',        AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(PivotRootRule.match(rawrule))  # the above invalid rules still match the main regex!
        with self.assertRaises(expected):
            PivotRootRule.create_instance(rawrule)

    def test_invalid_rule_name(self):
        self.assertFalse(PivotRootRule.match('pivot_rootbeer,'))
        with self.assertRaises(AppArmorException):
            PivotRootRule.create_instance('pivot_rootbeer,')


class PivotRootTestParseFromLog(PivotRootTest):
    def test_pivot_root_from_log(self):
        parser = ReadLog('', '', '')
        event = 'type=AVC msg=audit(1409700678.384:547594): apparmor="DENIED" operation="pivotroot" profile="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/pivot_root" name="/tmp/sdtest.21082-7446-EeefO6/new_root/" pid=21162 comm="pivot_root" srcname="/tmp/sdtest.21082-7446-EeefO6/new_root/put_old/"'

        parsed_event = parser.parse_event(event)

        self.assertEqual(parsed_event, {
            'request_mask': None,
            'denied_mask': None,
            'error_code': 0,
            'magic_token': 0,
            'parent': 0,
            'profile': '/home/ubuntu/bzr/apparmor/tests/regression/apparmor/pivot_root',
            'operation': 'pivotroot',
            'resource': None,
            'info': None,
            'aamode': 'REJECTING',
            'time': 1409700678,
            'active_hat': None,
            'pid': 21162,
            'task': 0,
            'attr': None,
            'name2': None,
            'src_name': '/tmp/sdtest.21082-7446-EeefO6/new_root/put_old/',
            'name': '/tmp/sdtest.21082-7446-EeefO6/new_root/',
            'family': None,
            'protocol': None,
            'sock_type': None,
            'class': None,
        })

        obj = PivotRootRule(parsed_event['src_name'], parsed_event['name'], PivotRootRule.ALL, log_event=parsed_event)

        #             audit  allow  deny  comment   oldroot                                            all?   newroot                                    all?   target all?
        expected = exp(False, False, False, '',     '/tmp/sdtest.21082-7446-EeefO6/new_root/put_old/', False, '/tmp/sdtest.21082-7446-EeefO6/new_root/', False, None, True)

        self._compare_obj(obj, expected)

        self.assertEqual(
            obj.get_raw(1),
            '  pivot_root oldroot=/tmp/sdtest.21082-7446-EeefO6/new_root/put_old/ /tmp/sdtest.21082-7446-EeefO6/new_root/,')


class PivotRootFromInit(PivotRootTest):
    tests = (
        # PivotRootRule object                                                                  audit  allow  deny   comment    oldroot     all?    newroot         all?    profile_name    all?
        (PivotRootRule('/oldroot',        '/new/root',       'some_profile',    deny=True), exp(False, False, True,  '',        '/oldroot', False,  '/new/root',    False,  'some_profile', False)),
        (PivotRootRule('/oldroot',        '/new/root',       PivotRootRule.ALL, deny=True), exp(False, False, True,  '',        '/oldroot', False,  '/new/root',    False,  None,           True)),
        (PivotRootRule('/oldroot',        PivotRootRule.ALL, '/someprofile',    deny=True), exp(False, False, True,  '',        '/oldroot', False,  None,           True,   '/someprofile', False)),
        (PivotRootRule(PivotRootRule.ALL, '/new/root',       '/someprofile',    deny=True), exp(False, False, True,  '',        None,       True,   '/new/root',    False,  '/someprofile', False)),
        (PivotRootRule('/oldroot',        PivotRootRule.ALL, PivotRootRule.ALL, deny=True), exp(False, False, True,  '',        '/oldroot', False,  None,           True,   None,           True)),
        (PivotRootRule(PivotRootRule.ALL, '/new/root',       PivotRootRule.ALL, deny=True), exp(False, False, True,  '',        None,       True,   '/new/root',    False,  None,           True)),
        (PivotRootRule(PivotRootRule.ALL, PivotRootRule.ALL, 'some_profile',    deny=True), exp(False, False, True,  '',        None,       True,   None,           True,   'some_profile', False)),
        (PivotRootRule(PivotRootRule.ALL, PivotRootRule.ALL, PivotRootRule.ALL, deny=True), exp(False, False, True,  '',        None,       True,   None,           True,   None,           True)),
    )

    def _run_test(self, obj, expected):
        self._compare_obj(obj, expected)


class InvalidPivotRootInit(AATest):
    tests = (
        # (init params, expected exception)
        (('',     '/foo', 'bar'), AppArmorBug),        # empty oldroot
        (('/old', '',     'bar'), AppArmorBug),        # empty newroot
        (('/old', '/foo', ''   ), AppArmorBug),        # empty targetprof # noqa: E202

        (('old',  '/foo', 'bar'), AppArmorException),  # oldroot is not a path
        (('/old', 'foo',  'bar'), AppArmorException),  # newroot is not a path


        ((None,   '/foo', 'bar'), AppArmorBug),        # wrong type
        (('/old', None,   'bar'), AppArmorBug),        #
        (('/old', '/foo', None ), AppArmorBug),                           # noqa: E202

        ((dict(), '/foo', 'bar'), AppArmorBug),        # wrong type
        (('/old', dict(), 'bar'), AppArmorBug),        #
        (('/old', '/foo', dict()), AppArmorBug),       #
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            PivotRootRule(*params)

    def test_missing_params_1(self):
        with self.assertRaises(TypeError):
            PivotRootRule()

    def test_missing_params_2(self):
        with self.assertRaises(TypeError):
            PivotRootRule('/foo')

    def test_missing_params_3(self):
        with self.assertRaises(TypeError):
            PivotRootRule('/foo', '/bar')


class InvalidPivotRootTest(AATest):
    def _check_invalid_rawrule(self, rawrule):
        obj = None
        self.assertFalse(PivotRootRule.match(rawrule))
        with self.assertRaises(AppArmorException):
            obj = PivotRootRule.create_instance(rawrule)

        self.assertIsNone(obj, 'PivotRootRule handed back an object unexpectedly')

    def test_invalid_pivot_root_missing_comma(self):
        self._check_invalid_rawrule('pivot_root')  # missing comma

    def test_invalid_non_PivotRootRule(self):
        self._check_invalid_rawrule('dbus,')  # not a pivot_root rule

    def test_empty_data_1(self):
        obj = PivotRootRule('/foo', '/bar', 'prof')
        obj.oldroot = ''
        # no oldroot set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)

    def test_empty_data_2(self):
        obj = PivotRootRule('/foo', '/bar', 'prof')
        obj.newroot = ''
        # no newroot set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)

    def test_empty_data_3(self):
        obj = PivotRootRule('/foo', '/bar', 'prof')
        obj.profile_name = ''
        # no profile_name set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)


class WritePivotRootTestAATest(AATest):
    def _run_test(self, rawrule, expected):
        self.assertTrue(PivotRootRule.match(rawrule))
        obj = PivotRootRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    tests = (
        #  raw rule                                           clean rule
        ('pivot_root,',                                           'pivot_root,'),
        ('     pivot_root         ,    # foo        ',            'pivot_root, # foo'),
        ('    audit     pivot_root /foo,',                        'audit pivot_root /foo,'),
        ('   deny pivot_root         /foo      ,# foo bar',       'deny pivot_root /foo, # foo bar'),
        ('   deny pivot_root        "/foo"     ,# foo bar',       'deny pivot_root /foo, # foo bar'),
        ('   allow pivot_root                        ,# foo bar', 'allow pivot_root, # foo bar'),
        ('   pivot_root  oldroot=/old       ,    # foo        ',            'pivot_root oldroot=/old, # foo'),
        ('   pivot_root  oldroot="/old"     ,    # foo        ',            'pivot_root oldroot=/old, # foo'),
        ('   pivot_root  oldroot=/old    ->  some_profile ,   ',            'pivot_root oldroot=/old -> some_profile,'),
        ('   pivot_root  oldroot=/old /new   ->  some_profile ,   ',            'pivot_root oldroot=/old /new -> some_profile,'),
        ('priority=1 pivot_root  oldroot=/old /new   ->  some_profile ,   ',    'priority=1 pivot_root oldroot=/old /new -> some_profile,'),
        ('priority=0 pivot_root  oldroot=/old /new   ->  some_profile ,   ',    'priority=0 pivot_root oldroot=/old /new -> some_profile,'),
        ('priority=-1 pivot_root  oldroot=/old /new   ->  some_profile ,  ',    'priority=-1 pivot_root oldroot=/old /new -> some_profile,'),
        ('priority=+1 pivot_root  oldroot=/old /new   ->  some_profile ,  ',    'priority=1 pivot_root oldroot=/old /new -> some_profile,'),
    )

    def test_write_manually(self):
        obj = PivotRootRule('/old', '/new', 'target', allow_keyword=True)

        expected = '    allow pivot_root oldroot=/old /new -> target,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')


class PivotRootCoveredTest(AATest):
    def _run_test(self, param, expected):
        obj = PivotRootRule.create_instance(self.rule)
        check_obj = PivotRootRule.create_instance(param)

        self.assertTrue(PivotRootRule.match(param))

        self.assertEqual(obj.is_equal(check_obj), expected[0], 'Mismatch in is_equal, expected {}'.format(expected[0]))
        self.assertEqual(obj.is_equal(check_obj, True), expected[1], 'Mismatch in is_equal/strict, expected {}'.format(expected[1]))

        self.assertEqual(obj.is_covered(check_obj), expected[2], 'Mismatch in is_covered, expected {}'.format(expected[2]))
        self.assertEqual(obj.is_covered(check_obj, True, True), expected[3], 'Mismatch in is_covered/exact, expected {}'.format(expected[3]))


class PivotRootCoveredTest_01(PivotRootCoveredTest):
    rule = 'pivot_root /new,'

    tests = (
        #   rule                                         equal  strict equal  covered  covered exact
        ('pivot_root,',                                 (False, False,        False,   False)),
        ('pivot_root              /n*,',                (False, False,        False,   False)),
        ('pivot_root oldroot=/old,',                    (False, False,        False,   False)),
        ('pivot_root              /new,',               (True,  False,        True,    True)),
        ('pivot_root                   -> target,',     (False, False,        False,   False)),
        ('pivot_root oldroot=/old /new,',               (False, False,        True,    True)),
        ('pivot_root              /new -> target,',     (False, False,        True,    True)),
        ('pivot_root oldroot=/old      -> target,',     (False, False,        False,   False)),
        ('pivot_root oldroot=/old /new -> target,',     (False, False,        True,    True)),
    )


class PivotRootCoveredTest_02(PivotRootCoveredTest):
    rule = 'audit pivot_root oldroot=/ol*,'

    tests = (
        #   rule                                               equal  strict equal  covered  covered exact
        ('audit pivot_root,',                                 (False, False,        False,   False)),
        ('audit pivot_root oldroot=/ol*,',                    (True,  True,         True,    True)),
        ('audit pivot_root oldroot=/old,',                    (False, False,        True,    True)),
        ('audit pivot_root              /new,',               (False, False,        False,   False)),
        ('audit pivot_root                   -> target,',     (False, False,        False,   False)),
        ('audit pivot_root oldroot=/old /new,',               (False, False,        True,    True)),
        ('audit pivot_root              /new -> target,',     (False, False,        False,   False)),
        ('audit pivot_root oldroot=/old      -> target,',     (False, False,        True,    True)),  # covered exact - really?
        ('audit pivot_root oldroot=/old /new -> target,',     (False, False,        True,    True)),  # covered exact - really?
    )


class PivotRootCoveredTest_03(PivotRootCoveredTest):
    rule = 'pivot_root -> target,'

    tests = (
        #   rule                                         equal  strict equal  covered  covered exact
        ('pivot_root,',                                 (False, False,        False,   False)),
        ('pivot_root oldroot=/ol*,',                    (False, False,        False,   False)),
        ('pivot_root oldroot=/old,',                    (False, False,        False,   False)),
        ('pivot_root              /new,',               (False, False,        False,   False)),
        ('pivot_root                   -> target,',     (True,  False,        True,    True)),
        ('pivot_root oldroot=/old /new,',               (False, False,        False,   False)),
        ('pivot_root              /new -> target,',     (False, False,        True,    True)),
        ('pivot_root oldroot=/old      -> target,',     (False, False,        True,    True)),
        ('pivot_root oldroot=/old /new -> target,',     (False, False,        True,    True)),
    )


class PivotRootCoveredTest_04(PivotRootCoveredTest):
    rule = 'deny pivot_root /foo,'

    tests = (
        #   rule                         equal  strict equal  covered   covered exact
        ('      deny pivot_root /foo,',     (True,  True,         True,    True)),
        ('audit deny pivot_root /foo,',     (False, False,        False,   False)),
        ('           pivot_root /foo,',     (False, False,        False,   False)),  # XXX should covered be true here?
        ('      deny pivot_root /bar,',     (False, False,        False,   False)),
        ('      deny pivot_root,',          (False, False,        False,   False)),
    )


class PivotRootCoveredTest_Invalid(AATest):
    # TODO: should this be detected?
    # def test_borked_obj_is_covered_1(self):
    #     obj = PivotRootRule.create_instance('pivot_root oldroot=/old /new -> target,')

    #     testobj = PivotRootRule('/old', '/foo', 'targetprof')
    #     testobj.oldrooot = None

    #     with self.assertRaises(AppArmorBug):
    #         obj.is_covered(testobj)

    def test_borked_obj_is_covered_2(self):
        obj = PivotRootRule.create_instance('pivot_root oldroot=/old /new -> target,')

        testobj = PivotRootRule('/old', '/foo', 'targetprof')
        testobj.newroot = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    # def test_borked_obj_is_covered_3(self):
    # TODO: should this be detected?
    #     obj = PivotRootRule.create_instance('pivot_root oldroot=/old /new -> target,')

    #     testobj = PivotRootRule('/old', '/foo', 'targetprof')
    #     testobj.profile_name = ''

    #     with self.assertRaises(AppArmorBug):
    #         obj.is_covered(testobj)

    def test_invalid_is_covered(self):
        raw_rule = 'pivot_root oldroot=/old /new -> target,'

        class SomeOtherClass(PivotRootRule):
            pass

        obj = PivotRootRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_equal_1(self):
        raw_rule = 'pivot_root oldroot=/old /new -> target,'

        class SomeOtherClass(PivotRootRule):
            pass

        obj = PivotRootRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)

#   def test_invalid_is_equal_2(self):
    # TODO: should this be detected?
#       obj = PivotRootRule.create_instance('pivot_root oldroot=/old /new -> target,')

#       testobj = PivotRootRule.create_instance('pivot_root oldroot=/old /new -> target,')
#       testobj.all_oldroots = False  # make testobj invalid (should trigger exception in _is_equal_aare())

#       with self.assertRaises(AppArmorBug):
#           obj.is_equal(testobj)


class PivotRootLogprofHeaderTest(AATest):
    tests = (
        ('pivot_root,',                             [                              _('Old root'), _('ALL'),  _('New root'), _('ALL'),  _('Target profile'), _('ALL')]),  # noqa: E201
        ('pivot_root oldroot=/old,',                [                              _('Old root'), '/old',    _('New root'), _('ALL'),  _('Target profile'), _('ALL')]),  # noqa: E201
        ('deny pivot_root,',                        [_('Qualifier'), 'deny',       _('Old root'), _('ALL'),  _('New root'), _('ALL'),  _('Target profile'), _('ALL')]),
        ('allow pivot_root oldroot=/old,',          [_('Qualifier'), 'allow',      _('Old root'), '/old',    _('New root'), _('ALL'),  _('Target profile'), _('ALL')]),
        ('audit pivot_root /new,',                  [_('Qualifier'), 'audit',      _('Old root'), _('ALL'),  _('New root'), '/new',    _('Target profile'), _('ALL')]),
        ('audit deny pivot_root /new -> target,',   [_('Qualifier'), 'audit deny', _('Old root'), _('ALL'),  _('New root'), '/new',    _('Target profile'), 'target']),
        ('pivot_root oldroot=/old /new -> target,', [                              _('Old root'), '/old',    _('New root'), '/new',    _('Target profile'), 'target']),  # noqa: E201
    )

    def _run_test(self, params, expected):
        obj = PivotRootRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


class PivotRootEditHeaderTest(AATest):
    def _run_test(self, params, expected):
        rule_obj = PivotRootRule.create_instance(params)
        self.assertEqual(rule_obj.can_edit, True)
        prompt, path_to_edit = rule_obj.edit_header()
        self.assertEqual(path_to_edit, expected)

    tests = (
        ('pivot_root oldroot=/old /foo/bar/baz -> target,',     '/foo/bar/baz'),
        ('pivot_root /foo/**/baz,',                             '/foo/**/baz'),
        ('pivot_root /foo/** -> /bar,',                         '/foo/**'),
    )

    def test_edit_header_bare_pivot_root(self):
        rule_obj = PivotRootRule.create_instance('pivot_root,')
        self.assertEqual(rule_obj.can_edit, False)
        with self.assertRaises(AppArmorBug):
            rule_obj.edit_header()


class PivotRootValidateAndStoreEditTest(AATest):
    def _run_test(self, params, expected):
        rule_obj = PivotRootRule('/old/', '/foo/bar/baz', 'target', log_event=True)

        self.assertEqual(rule_obj.validate_edit(params), expected)

        rule_obj.store_edit(params)
        self.assertEqual(rule_obj.get_raw(), 'pivot_root oldroot=/old/ ' + params + ' -> target,')

    tests = (
        # edited path     match
        ('/foo/bar/baz',  True),
        ('/foo/bar/*',    True),
        ('/foo/bar/???',  True),
        ('/foo/xy**',     False),
        ('/foo/bar/baz/', False),
    )

    def test_validate_not_a_path(self):
        rule_obj = PivotRootRule.create_instance('pivot_root /foo/bar/baz,')

        with self.assertRaises(AppArmorException):
            rule_obj.validate_edit('foo/bar/baz')

        with self.assertRaises(AppArmorException):
            rule_obj.store_edit('foo/bar/baz')

    def test_validate_edit_bare_pivot_root(self):
        rule_obj = PivotRootRule.create_instance('pivot_root,')
        self.assertEqual(rule_obj.can_edit, False)

        with self.assertRaises(AppArmorBug):
            rule_obj.validate_edit('/foo/bar')

        with self.assertRaises(AppArmorBug):
            rule_obj.store_edit('/foo/bar')


# --- tests for PivotRootRuleset --- #

class PivotRootRulesTest(AATest):
    def test_empty_ruleset(self):
        ruleset = PivotRootRuleset()
        ruleset_2 = PivotRootRuleset()
        self.assertEqual([], ruleset.get_raw(2))
        self.assertEqual([], ruleset.get_clean(2))
        self.assertEqual([], ruleset_2.get_raw(2))
        self.assertEqual([], ruleset_2.get_clean(2))

        # test __repr__() for empty ruleset
        self.assertEqual(str(ruleset), '<PivotRootRuleset (empty) />')

    def test_ruleset_1(self):
        ruleset = PivotRootRuleset()
        rules = (
            'pivot_root oldroot=/foo,',
            'pivot_root /new,',
        )

        expected_raw = [
            'pivot_root oldroot=/foo,',
            'pivot_root /new,',
            '',
        ]

        expected_clean = [
            'pivot_root /new,',
            'pivot_root oldroot=/foo,',
            '',
        ]

        for rule in rules:
            ruleset.add(PivotRootRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw())
        self.assertEqual(expected_clean, ruleset.get_clean())

        # test __repr__() for non-empty ruleset
        self.assertEqual(
            str(ruleset), '<PivotRootRuleset>\n  pivot_root oldroot=/foo,\n  pivot_root /new,\n</PivotRootRuleset>')


class PivotRootGlobTestAATest(AATest):
    def test_glob(self):
        glob_list = [(
            'pivot_root /foo/bar,',
            'pivot_root /foo/*,',
            'pivot_root /**,',
        )]
        for globs in glob_list:
            for i in range(len(globs) - 1):
                rule = PivotRootRule.create_instance(globs[i])
                rule.glob()
                self.assertEqual(rule.get_clean(), globs[i + 1])

    def test_glob_all(self):
        glob_list = [(
            'pivot_root,',
            'pivot_root,',
        )]
        for globs in glob_list:
            for i in range(len(globs) - 1):
                rule = PivotRootRule.create_instance(globs[i])
                rule.glob()
                self.assertEqual(rule.get_clean(), globs[i + 1])


#   def test_glob_ext(self):
#       # rule = PivotRootRule.create_instance('pivot_root /foo/bar,')
#       with self.assertRaises(NotImplementedError):
#           # get_glob_ext is not available for pivot_root rules
#           self.ruleset.get_glob_ext('pivot_root /foo,')


# class PivotRootDeleteTestAATest(AATest):
#     pass


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)
